@@ -31,11 +31,10 @@
 #include <k3listview.h>
 #include <kdialog.h>
 
-#include <QLayout>
+#include <QBoxLayout>
 #include <QDir>
-#include <QPixmap>
-#include <QImage>
 #include <QLabel>
+#include <QPixmap>
 
 #include "themepage.h"
 #include "themepage.moc"
@@ -63,6 +62,7 @@ ThemePage::ThemePage( QWidget* parent, const char* name )
 	listview = new K3ListView( this );
 	listview->setFullWidth( true );
 	listview->setAllColumnsShowFocus( true );
+	listview->setSelectionModeExt( K3ListView::Single );
 	listview->addColumn( i18n("Name") );
 	listview->addColumn( i18n("Description") );
 	layout->addWidget(listview);
