@@ -35,6 +35,7 @@
 #include <KUrlRequester>
 #include <KUrlRequesterDialog>
 #include <knewstuff2/engine.h>
+#include <KDebug>
 
 #include <QDir>
 #include <QGridLayout>
@@ -65,11 +66,14 @@ KDMThemeWidget::KDMThemeWidget( QWidget *parent )
 	: QWidget( parent )
 {
 	QGridLayout *ml = new QGridLayout( this );
+	ml->setSizeConstraint( QLayout::SetMinAndMaxSize );
 	ml->setSpacing( KDialog::spacingHint() );
 	ml->setMargin( KDialog::marginHint() );
 
 	themeWidget = new QTreeWidget( this );
-	themeWidget->setHeaderLabels( QStringList() << i18n("Theme") << i18n("Author") );
+	themeWidget->setHeaderLabels( QStringList()
+		<< i18nc("@title:column", "Theme")
+		<< i18nc("@title:column", "Author") );
 	themeWidget->setSortingEnabled( true );
 	themeWidget->sortItems( 0, Qt::AscendingOrder );
 	themeWidget->setRootIsDecorated( false );
@@ -93,18 +97,17 @@ KDMThemeWidget::KDMThemeWidget( QWidget *parent )
 
 	ml->addWidget( info, 1, 4 );
 
-	bInstallTheme = new QPushButton( i18n("Install &new theme"), this );
+	bInstallTheme = new QPushButton( i18nc("@action:button", "Install &new theme"), this );
 	bInstallTheme->setWhatsThis( i18n("This will install a theme into the theme directory.") );
 
 	ml->addWidget( bInstallTheme, 2, 0 );
 
-	bRemoveTheme = new QPushButton( i18n("&Remove theme"), this );
+	bRemoveTheme = new QPushButton( i18nc("@action:button", "&Remove theme"), this );
 	bRemoveTheme->setWhatsThis( i18n("This will remove the selected theme.") );
 
 	ml->addWidget( bRemoveTheme, 2, 1 );
 
-	bGetNewThemes = new QPushButton( i18n("&Get New Themes"), this );
-	bGetNewThemes->setWhatsThis( i18n("Get New Themes") );
+	bGetNewThemes = new QPushButton( i18nc("@action:button", "&Get New Themes"), this );
 
 	ml->addWidget( bGetNewThemes, 2, 2 );
 
@@ -113,13 +116,13 @@ KDMThemeWidget::KDMThemeWidget( QWidget *parent )
 	connect( bRemoveTheme, SIGNAL(clicked()), SLOT(removeSelectedThemes()) );
 	connect( bGetNewThemes, SIGNAL(clicked()), SLOT(getNewStuff()) );
 
-	themeDir = KGlobal::dirs()->resourceDirs( "data" ).last() + "kdm/themes/";
+	themeDir = KStandardDirs::installPath( "data" ) + "kdm/themes/";
 	defaultTheme = 0;
 	QDir testDir( themeDir );
 	if (!testDir.exists() && !testDir.mkdir( testDir.absolutePath() ) && !geteuid())
 		KMessageBox::sorry( this, i18n("Unable to create folder %1", testDir.absolutePath() ) );
 
-	foreach (QString ent,
+	foreach (const QString& ent,
 	         QDir( themeDir ).entryList( QDir::Dirs | QDir::NoDotAndDotDot,
 	                                     QDir::Unsorted ))
 		insertTheme( themeDir + ent );
@@ -139,17 +142,17 @@ void KDMThemeWidget::selectTheme( const QString &path )
 
 void KDMThemeWidget::load()
 {
-	selectTheme( config->group("X-*-Greeter").readEntry( "Theme", themeDir + "circles" ) );
+	selectTheme( config->group( "X-*-Greeter" ).readEntry( "Theme", themeDir + "oxygen" ) );
 }
 
 void KDMThemeWidget::save()
 {
-	config->group("X-*-Greeter").writeEntry( "Theme", defaultTheme ? defaultTheme->path : "" );
+	config->group( "X-*-Greeter" ).writeEntry( "Theme", defaultTheme ? defaultTheme->path : "" );
 }
 
 void KDMThemeWidget::defaults()
 {
-	selectTheme( themeDir + "circles" );
+	selectTheme( themeDir + "oxygen" );
 
 	emit changed();
 }
@@ -164,8 +167,8 @@ void KDMThemeWidget::makeReadOnly()
 
 void KDMThemeWidget::insertTheme( const QString &_theme )
 {
-	KConfig themeConfig( _theme + "/KdmGreeterTheme.desktop", KConfig::SimpleConfig);
-	KConfigGroup themeGroup = themeConfig.group("KdmGreeterTheme");
+	KConfig themeConfig( _theme + "/KdmGreeterTheme.desktop", KConfig::SimpleConfig );
+	KConfigGroup themeGroup = themeConfig.group( "KdmGreeterTheme" );
 
 	QString name = themeGroup.readEntry( "Name" );
 	if (name.isEmpty())
@@ -180,6 +183,16 @@ void KDMThemeWidget::insertTheme( const QString &_theme )
 	child->description = themeGroup.readEntry( "Description" );
 }
 
+void KDMThemeWidget::removeTheme( const QString &name )
+{
+	if (name.isEmpty())
+		return;
+
+	QList<QTreeWidgetItem *> ls = themeWidget->findItems( name, Qt::MatchExactly );
+	if (!ls.isEmpty())
+		delete ls.first();
+}
+
 void KDMThemeWidget::updateInfoView( ThemeData *theme )
 {
 	if (!(defaultTheme = theme)) {
@@ -190,10 +203,10 @@ void KDMThemeWidget::updateInfoView( ThemeData *theme )
 		info->setText(
 			((theme->copyright.length() > 0) ?
 				i18n("<qt><strong>Copyright:</strong> %1<br/></qt>",
-					theme->copyright) : "") +
+				     theme->copyright) : "") +
 			((theme->description.length() > 0) ?
 				i18n("<qt><strong>Description:</strong> %1</qt>",
-					theme->description) : "") );
+				     theme->description) : "") );
 		preview->setPixmap( theme->path + '/' + theme->screenShot );
 		preview->setText( theme->screenShot.isEmpty() ?
 			"Screenshot not available" : QString() );
@@ -211,8 +224,8 @@ void KDMThemeWidget::installNewTheme()
 		return;
 
 #if 0
-	if (themeURL.isLocalFile() && QDir( themeURL.path() ).exists()) {
-		insertTheme( themeURL.path() );
+	if (themeURL.isLocalFile() && QDir( themeURL.toLocalFile() ).exists()) {
+		insertTheme( themeURL.toLocalFile() );
 		emit changed();
 		return;
 	}
@@ -234,10 +247,10 @@ void KDMThemeWidget::installNewTheme()
 	QList<const KArchiveDirectory *> foundThemes;
 
 	KTar archive( themeTmpFile );
-	archive.open( IO_ReadOnly );
+	archive.open( QIODevice::ReadOnly );
 
 	const KArchiveDirectory *archDir = archive.directory();
-	foreach (QString ent, archDir->entries()) {
+	foreach (const QString& ent, archDir->entries()) {
 		const KArchiveEntry *possibleDir = archDir->entry( ent );
 		if (possibleDir->isDirectory()) {
 			const KArchiveDirectory *subDir =
@@ -251,7 +264,7 @@ void KDMThemeWidget::installNewTheme()
 		KMessageBox::error( this, i18n("The file is not a valid KDM theme archive.") );
 	else {
 		KProgressDialog progressDiag( this,
-			i18n("Installing KDM themes"), QString() );
+			i18nc("@title:window", "Installing KDM themes"), QString() );
 		progressDiag.setModal( true );
 		progressDiag.setAutoClose( true );
 		progressDiag.progressBar()->setMaximum( foundThemes.size() );
@@ -259,7 +272,8 @@ void KDMThemeWidget::installNewTheme()
 
 		foreach (const KArchiveDirectory *ard, foundThemes) {
 			progressDiag.setLabelText(
-				i18n("<qt>Installing <strong>%1</strong> theme</qt>", ard->name() ) );
+				i18nc("@info:progress",
+				      "<qt>Installing <strong>%1</strong> theme</qt>", ard->name() ) );
 
 			QString path = themeDir + ard->name();
 			ard->copyTo( path, true );
@@ -284,6 +298,8 @@ void KDMThemeWidget::themeSelected()
 		updateInfoView( (ThemeData *)(themeWidget->selectedItems().first()) );
 	else
 		updateInfoView( 0 );
+	if (bInstallTheme->isEnabled()) // not read-only
+		bRemoveTheme->setEnabled( !themeWidget->selectedItems().isEmpty() );
 	emit changed();
 }
 
@@ -298,8 +314,8 @@ void KDMThemeWidget::removeSelectedThemes()
 		delList.append( ((ThemeData *)itm)->path );
 	}
 	if (KMessageBox::questionYesNoList( this,
-			i18n("Are you sure you want to remove the following themes?"),
-			nameList, i18n("Remove themes?") ) != KMessageBox::Yes)
+	        i18n("Are you sure you want to remove the following themes?"),
+	        nameList, i18nc("@title:window", "Remove themes?") ) != KMessageBox::Yes)
 		return;
 	KIO::del( KUrl::List( delList ) ); // XXX error check
 
@@ -309,9 +325,23 @@ void KDMThemeWidget::removeSelectedThemes()
 
 void KDMThemeWidget::getNewStuff()
 {
-	KNS::Engine engine(this);
-	engine.init("kdm.knsrc");
-	KNS::Entry::List entries = engine.downloadDialogModal(this);
+	KNS::Engine engine( this );
+	if (engine.init( "kdm.knsrc" )) {
+		KNS::Entry::List entries = engine.downloadDialogModal( this );
+		for (int i = 0; i < entries.size(); i ++) {
+			if (entries.at( i )->status() == KNS::Entry::Installed) {
+				if (!entries.at( i )->installedFiles().isEmpty()) {
+					QString name = entries.at( i )->installedFiles().at( 0 ).section( '/', -2, -2 );
+					insertTheme( themeDir + name );
+				}
+			} else if (entries.at( i )->status() == KNS::Entry::Deleted) {
+				if (!entries.at( i )->uninstalledFiles().isEmpty()) {
+					QString name = entries.at( i )->uninstalledFiles().at( 0 ).section( '/', -2, -2 );
+					removeTheme( themeDir + name );
+				}
+			}
+		}
+	}
 }
 
 #include "kdm-theme.moc"
