@@ -63,6 +63,7 @@
 #include <KDE/KMenu>
 #include <KDE/KPluginFactory>
 #include <KDE/KPluginLoader>
+#include <KDE/KStandardAction>
 
 #define CFG_GROUP                  "Main Settings"
 #define CFG_PREVIEW_SPLITTER_SIZES "PreviewSplitterSizes"
@@ -79,7 +80,7 @@ K_EXPORT_PLUGIN(FontInstallFactory("fontinst"))
 namespace KFI
 {
 
-static int constModeList[]=
+static const int constModeList[]=
     {CGroupListItem::ALL, CGroupListItem::PERSONAL, CGroupListItem::SYSTEM, -1};
 
 inline CGroupListItem::EType modeToGrp(int mode)
@@ -212,14 +213,12 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
 {
     setButtons(Help);
 
-    CFcEngine::instance()->readConfig(itsConfig);
-    CFcEngine::setBgndCol(QApplication::palette().color(QPalette::Active, QPalette::Base));
-    CFcEngine::setTextCol(QApplication::palette().color(QPalette::Active, QPalette::Text));
     KGlobal::locale()->insertCatalog(KFI_CATALOGUE);
     KIconLoader::global()->addAppDir(KFI_NAME);
+    KIconLoader::global()->reconfigure(KFI_NAME, KGlobal::dirs());
 
     KAboutData *about = new KAboutData("fontinst", 0, ki18n("KDE Font Installer"), 0, KLocalizedString(),
-                                       KAboutData::License_GPL, ki18n("GUI front end to the fonts:/ ioslave.\n"
+                                       KAboutData::License_GPL, ki18n("GUI front-end to the fonts:/ ioslave.\n"
                                                                       "(c) Craig Drummond, 2000 - 2007"));
     about->addAuthor(ki18n("Craig Drummond"), ki18n("Developer and maintainer"), "craig@kde.org");
     setAboutData(about);
@@ -237,8 +236,8 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     itsGroupSplitter->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
 
     QStringList items;
-    QBoxLayout  *mainLayout=new QBoxLayout(QBoxLayout::TopToBottom, this),
-                *controlLayout=new QBoxLayout(QBoxLayout::TopToBottom, controls);
+    QBoxLayout  *mainLayout=new QBoxLayout(QBoxLayout::TopToBottom, this);
+    QGridLayout *controlLayout=new QGridLayout(controls);
 
     itsGroupsWidget=new QWidget(itsGroupSplitter);
     itsFontsWidget=new QWidget(itsGroupSplitter);
@@ -261,8 +260,12 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
 
     // Preview...
     itsPreviewControl=new CPreviewSelectAction(itsPreviewWidget);
-
+    itsZoomInAction=KStandardAction::zoomIn(this, SLOT(zoomIn()), this);
+    itsZoomOutAction=KStandardAction::zoomOut(this, SLOT(zoomOut()), this);
     previewToolbar->addAction(itsPreviewControl);
+    previewToolbar->addAction(itsZoomInAction);
+    previewToolbar->addAction(itsZoomOutAction);
+    previewToolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
 
     previewLayout->setMargin(0);
     previewLayout->setSpacing(KDialog::spacingHint());
@@ -281,10 +284,14 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     itsPreview->setContextMenuPolicy(Qt::CustomContextMenu);
     previewFrameLayout->addWidget(itsPreview);
 
+    itsPreview->engine()->readConfig(itsConfig);
+    connect(itsPreview, SIGNAL(doZoomIn()), SLOT(zoomIn()));
+    connect(itsPreview, SIGNAL(doZoomOut()), SLOT(zoomOut()));
+
     // Toolbar...
     KActionMenu *settingsMenu=new KActionMenu(KIcon("configure"), i18n("Settings"), this);
     KAction     *changeTextAct=new KAction(KIcon("edit-rename"), i18n("Change Preview Text..."), this),
-                *duplicateFontsAct=new KAction(KIcon("system-search"), i18n("Scan For Duplicate Fonts..."), this);
+                *duplicateFontsAct=new KAction(KIcon("system-search"), i18n("Scan for Duplicate Fonts..."), this);
                 //*validateFontsAct=new KAction(KIcon("checkmark"), i18n("Validate Fonts..."), this);
                 //*downloadFontsAct=new KAction(KIcon("go-down"), i18n("Download Fonts..."), this);
 
@@ -310,11 +317,9 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     {
         itsModeControl=new QComboBox(toolbar);
         itsModeAct=toolbar->addWidget(itsModeControl);
-        toolbar->addSeparator();
     }
 
-    itsFilter=new CFontFilter(toolbar);
-    toolbar->addWidget(itsFilter);
+    itsFilter=new CFontFilter(this);
 
     // Details - Groups...
     itsGroupList=new CGroupList(itsGroupsWidget);
@@ -348,7 +353,7 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     groupsLayout->addItem(new QSpacerItem(itsDisableGroupControl->width(), KDialog::spacingHint(),
                           QSizePolicy::Expanding, QSizePolicy::Fixed), 1, 4);
     // Details - Fonts...
-    itsFontList=new CFontList(itsFontsWidget);
+    itsFontList=new CFontList(itsPreview->engine(), itsFontsWidget);
     itsFontListView=new CFontListView(itsFontsWidget, itsFontList);
     itsFontListView->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     itsFontListView->readConfig(cg);
@@ -357,7 +362,7 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
                                                i18n("Install fonts")),
                                       itsFontsWidget);
 
-    itsDeleteFontControl=new CPushButton(KGuiItem(i18n("Delete..."), "edit-delete",
+    itsDeleteFontControl=new CPushButton(KGuiItem(i18n("Delete"), "edit-delete",
                                                   i18n("Delete all selected fonts")),
                                          itsFontsWidget);
 
@@ -392,8 +397,9 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     statusLayout->addWidget(itsListingProgress);
 
     // Layout widgets...
-    controlLayout->addWidget(toolbar);
-    controlLayout->addWidget(itsGroupSplitter);
+    controlLayout->addWidget(toolbar, 0, 0);
+    controlLayout->addWidget(itsFilter, 0, 1);
+    controlLayout->addWidget(itsGroupSplitter, 1, 0, 1, 2);
     mainLayout->addWidget(itsPreviewSplitter);
     mainLayout->addWidget(statusWidget);
     previewLayout->addWidget(previewToolbar);
@@ -407,8 +413,6 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     itsPreviewSplitter->setStretchFactor(0, 1);
     itsPreviewSplitter->setStretchFactor(1, 0);
 
-    toolbar->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
-
     // Set sizes for 3 views...
     QList<int> defaultSizes;
 
@@ -553,6 +557,8 @@ void CKCmFontInst::fontSelected(const QModelIndex &index, bool en, bool dis)
 
                 itsPreview->showFont(url, font->isEnabled() ? font->name() : QString(),
                                      font->styleInfo(), font->isEnabled() ? 1 : font->index());
+                itsZoomInAction->setEnabled(!itsPreview->engine()->atMax());
+                itsZoomOutAction->setEnabled(!itsPreview->engine()->atMin());
             }
             itsDeleteFontControl->setEnabled(true);
         }
@@ -597,9 +603,9 @@ void CKCmFontInst::addFonts()
 
                     if(url.isLocalFile())
                     {
-                        QString file(url.path());
+                        QString file(url.toLocalFile());
 
-                        if(Misc::isPackage(file)) // If its a package we need to unzip 1st...
+                        if(Misc::isPackage(file)) // If it is a package we need to unzip 1st...
                         {
                             KZip zip(url.path());
 
@@ -749,7 +755,7 @@ void CKCmFontInst::print(bool all)
                                 str << (*it).family << endl
                                     << (*it).styleInfo << endl;
 
-                            args << "--embed" << QString().sprintf("0x%x", (unsigned int)topLevelWidget()->winId())
+                            args << "--embed" << QString().sprintf("0x%x", (unsigned int)window()->winId())
                                 << "--caption" << KGlobal::caption().toUtf8()
                                 << "--icon" << "preferences-desktop-font-installer"
                                 << "--size" << QString().setNum(constSizes[dlg.chosenSize() < 6 ? dlg.chosenSize() : 2])
@@ -764,7 +770,7 @@ void CKCmFontInst::print(bool all)
                     }
                     else
                     {
-                        args << "--embed" << QString().sprintf("0x%x", (unsigned int)topLevelWidget()->winId())
+                        args << "--embed" << QString().sprintf("0x%x", (unsigned int)window()->winId())
                             << "--caption" << KGlobal::caption().toUtf8()
                             << "--icon" << "preferences-desktop-font-installer"
                             << "--size" << QString().setNum(constSizes[dlg.chosenSize()<6 ? dlg.chosenSize() : 2]);
@@ -979,14 +985,14 @@ void CKCmFontInst::changeText()
     {
         bool             status;
         QRegExpValidator validator(QRegExp(".*"), 0L);
-        QString          oldStr(CFcEngine::instance()->getPreviewString()),
-                         newStr(KInputDialog::getText(i18n("Preview String"),
-                                                      i18n("Please enter new string:"),
+        QString          oldStr(itsPreview->engine()->getPreviewString()),
+                         newStr(KInputDialog::getText(i18n("Preview Text"),
+                                                      i18n("Please enter new text:"),
                                                       oldStr, &status, this, &validator));
 
         if(status && oldStr!=newStr)
         {
-            CFcEngine::instance()->setPreviewString(newStr);
+            itsPreview->engine()->setPreviewString(newStr);
             itsFontList->forceNewPreviews();
 
             if(itsPreview->width()>6)
@@ -1156,8 +1162,8 @@ void CKCmFontInst::addFonts(const QSet<KUrl> &src)
                 case CGroupListItem::UNCLASSIFIED:
                     switch(KMessageBox::questionYesNoCancel(this,
                            i18n("Do you wish to install the font(s) for personal use "
-                                "(only usable by you), or "
-                                "system-wide (usable by all users)?"),
+                                "(only available to you), or "
+                                "system-wide (available to all users)?"),
                            i18n("Where to Install"), KGuiItem(i18n(KFI_KIO_FONTS_USER)),
                                 KGuiItem(i18n(KFI_KIO_FONTS_SYS))))
                     {
@@ -1270,7 +1276,13 @@ void CKCmFontInst::toggleFontManagement(bool on)
         itsFontListView->setMgtMode(on);
         itsFilter->setMgtMode(on);
         if(itsModeControl)
+        {
             itsModeAct->setVisible(!on);
+            // The setEnabled() should not be required, as the widget is hidden. But, when the kcm switches
+            // out of management mode, and re-shows the mode combo, the combo for some reason is disabled.
+            // Calling setEnabled() here fixes this.
+            itsModeAct->setEnabled(!on);
+        }
         itsEnableFontControl->setVisible(on);
         itsDisableFontControl->setVisible(on);
         selectMainGroup();
@@ -1284,6 +1296,20 @@ void CKCmFontInst::selectMode(int mode)
     selectGroup(modeToGrp(mode));
 }
 
+void CKCmFontInst::zoomIn()
+{
+    itsPreview->zoomIn();
+    itsZoomInAction->setEnabled(!itsPreview->engine()->atMax());
+    itsZoomOutAction->setEnabled(!itsPreview->engine()->atMin());
+}
+
+void CKCmFontInst::zoomOut()
+{
+    itsPreview->zoomOut();
+    itsZoomInAction->setEnabled(!itsPreview->engine()->atMax());
+    itsZoomOutAction->setEnabled(!itsPreview->engine()->atMin());
+}
+
 void CKCmFontInst::selectGroup(CGroupListItem::EType grp)
 {
     QModelIndex current(itsGroupListView->currentIndex());
@@ -1366,8 +1392,8 @@ void CKCmFontInst::toggleFonts(CJobRunner::ItemList &urls, const QStringList &fo
                                             "contained within group \'<b>%2</b>\'?</p>",
                                             fonts.first(), grp),
                        enable ? i18n("Enable Font") : i18n("Disable Font"),
-                       enable ? KGuiItem(i18n("Enable"), "enablefont", i18n("Enable font"))
-                              : KGuiItem(i18n("Disable"), "disablefont", i18n("Disable font")));
+                       enable ? KGuiItem(i18n("Enable"), "enablefont", i18n("Enable Font"))
+                              : KGuiItem(i18n("Disable"), "disablefont", i18n("Disable Font")));
             break;
         default:
             doIt = KMessageBox::Yes==KMessageBox::warningYesNoList(this,
@@ -1390,8 +1416,8 @@ void CKCmFontInst::toggleFonts(CJobRunner::ItemList &urls, const QStringList &fo
                                              urls.count(), grp),
                        fonts,
                        enable ? i18n("Enable Fonts") : i18n("Disable Fonts"),
-                       enable ? KGuiItem(i18n("Enable"), "enablefont", i18n("Enable fonts"))
-                              : KGuiItem(i18n("Disable"), "disablefont", i18n("Disable fonts")));
+                       enable ? KGuiItem(i18n("Enable"), "enablefont", i18n("Enable Fonts"))
+                              : KGuiItem(i18n("Disable"), "disablefont", i18n("Disable Fonts")));
     }
 
     if(doIt && (!hasSys || itsRunner->getAdminPasswd(this)))
@@ -1411,7 +1437,7 @@ return false;
     if(NULL!=itsJob || itsFontList->active())
     {
         if(displayMsg)
-            KMessageBox::error(this, i18n("Sorry, the installer is currently busy. Please wait for the "
+            KMessageBox::error(this, i18n("The installer is currently busy. Please wait for the "
                                           "current operation to terminate."));
         return true;
     }
@@ -1442,7 +1468,7 @@ void CKCmFontInst::doCmd(CJobRunner::ECommand cmd, const CJobRunner::ItemList &u
 {
     itsFontList->setAutoUpdate(false);
     itsRunner->exec(cmd, urls, dest);
-    CFcEngine::instance()->setDirty();
+    CFcEngine::setDirty();
     setStatusBar();
     itsFontList->scan();
     itsFontList->setAutoUpdate(true);
