@@ -26,6 +26,8 @@
 #include "FcEngine.h"
 #include <QtGui/QImage>
 #include <QtGui/QPixmap>
+#include <QtGui/QApplication>
+#include <QtGui/QPalette>
 #include <QtCore/QFile>
 #include <KDE/KGlobalSettings>
 #include <KDE/KLocale>
@@ -56,17 +58,13 @@ CFontThumbnail::CFontThumbnail()
 
 bool CFontThumbnail::create(const QString &path, int width, int height, QImage &img)
 {
-    QPixmap  pix;
     QString  realPath(path);
     KTempDir *tempDir = 0;
 
-    CFcEngine::setBgndCol(Qt::white);
-    CFcEngine::setTextCol(Qt::black);
-
     KFI_DBUG << "Create font thumbnail for:" << path << endl;
 
     // Is this a fonts/package file? If so, extract 1 scalable font...
-    if(Misc::isPackage(path))
+    if(Misc::isPackage(path) || "application/zip"==KMimeType::findByFileContent(path)->name())
     {
         KZip zip(path);
 
@@ -112,28 +110,12 @@ bool CFontThumbnail::create(const QString &path, int width, int height, QImage &
         }
     }
 
-    if(CFcEngine::instance()->draw(KUrl(realPath), width, height, pix, 0, true))
-    {
-        img=pix.toImage().convertToFormat(QImage::Format_ARGB32);
-
-        int pixelsPerLine=img.bytesPerLine()/4;
-
-        for(int l=0; l<img.height(); ++l)
-        {
-            QRgb *scanLine=(QRgb *)img.scanLine(l);
-
-            for(int pixel=0; pixel<pixelsPerLine; ++pixel)
-                scanLine[pixel]=qRgba(qRed(scanLine[pixel]), qGreen(scanLine[pixel]),
-                                      qBlue(scanLine[pixel]),
-                                      0xFF-qRed(scanLine[pixel]));
-        }
-
-        delete tempDir;
-        return true;
-    }
+    QColor bgnd(Qt::black);
 
+    bgnd.setAlpha(0);
+    img=itsEngine.draw(KUrl(realPath), width, height, QApplication::palette().text().color(), bgnd, 0, true);
     delete tempDir;
-    return false;
+    return !img.isNull();
 }
 
 ThumbCreator::Flags CFontThumbnail::flags() const
