@@ -81,22 +81,30 @@ KCMXinerama::KCMXinerama(QWidget *parent, const QVariantList &)
 		xw = new XineramaWidget(this);
 		grid->addWidget(xw, 0, 0);
 
-		xw->headTable->setNumRows(_displays);
+		xw->headTable->setRowCount(_displays);
 
 		for (int i = 0; i < _displays; i++) {
 			QString l = i18n("Display %1", i+1);
 			QRect geom = QApplication::desktop()->screenGeometry(i);
 			xw->_unmanagedDisplay->addItem(l);
 			dpyList.append(l);
-			xw->headTable->setText(i, 0, QString::number(geom.x()));
-			xw->headTable->setText(i, 1, QString::number(geom.y()));
-			xw->headTable->setText(i, 2, QString::number(geom.width()));
-			xw->headTable->setText(i, 3, QString::number(geom.height()));
+			QTableWidgetItem *item = new QTableWidgetItem(QString::number(geom.x()));
+			item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
+			xw->headTable->setItem(i, 0, item);
+			item = new QTableWidgetItem(QString::number(geom.y()));
+			item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
+			xw->headTable->setItem(i, 1, item);
+			item = new QTableWidgetItem(QString::number(geom.width()));
+			item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
+			xw->headTable->setItem(i, 2, item);
+			item = new QTableWidgetItem(QString::number(geom.height()));
+			item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
+			xw->headTable->setItem(i, 3, item);
 		}
 
 		xw->_unmanagedDisplay->addItem(i18n("Display Containing the Pointer"));
 
-		xw->headTable->setRowLabels(dpyList);
+		xw->headTable->setVerticalHeaderLabels(dpyList);
 
 		connect(xw->_unmanagedDisplay, SIGNAL(activated(int)),
 			this, SLOT(windowIndicator(int)));
@@ -106,12 +114,11 @@ KCMXinerama::KCMXinerama(QWidget *parent, const QVariantList &)
 		connect(xw, SIGNAL(configChanged()), this, SLOT(changed()));
 	} else { // no Xinerama
 		QLabel *ql = new QLabel(i18n("<qt><p>This module is only for configuring systems with a single desktop spread across multiple monitors. You do not appear to have this configuration.</p></qt>"), this);
+		ql->setWordWrap(true);
 		grid->addWidget(ql, 0, 0);
 	}
 
 	grid->activate();
-
-	load();
 }
 
 KCMXinerama::~KCMXinerama() {
@@ -213,7 +220,7 @@ void KCMXinerama::windowIndicator(int dpy) {
 }
 
 QWidget *KCMXinerama::indicator(int dpy) {
-	QLabel *si = new QLabel(QString::number(dpy+1), 0, "Screen Indicator", Qt::X11BypassWindowManagerHint);
+	QLabel *si = new QLabel(QString::number(dpy+1), 0, Qt::X11BypassWindowManagerHint);
 
 	QFont fnt = KGlobalSettings::generalFont();
 	fnt.setPixelSize(100);
