@@ -2,6 +2,7 @@
 
 Copyright 1988, 1998  The Open Group
 Copyright 2000-2004 Oswald Buddenhagen <ossi@kde.org>
+Copyright 2008 Juuso Alasuutari <juuso.alasuutari@gmail.com>
 
 Permission to use, copy, modify, distribute, and sell this software and its
 documentation for any purpose is hereby granted without fee, provided that
@@ -44,6 +45,7 @@ from the copyright holder.
 # include <rpc/rpc.h>
 # include <rpc/key_prot.h>
 extern int key_setnet( struct key_netstarg *arg );
+# include <X11/Xlib.h>
 #endif
 #ifdef K5AUTH
 # include <krb5/krb5.h>
@@ -77,6 +79,10 @@ extern int loginsuccess( const char *User, const char *Host, const char *Tty, ch
 /* for expiration */
 # include <time.h>
 #endif /* USE_PAM || _AIX */
+#ifdef HAVE_CKCONNECTOR
+# include <ck-connector.h>
+# include <dbus/dbus.h>
+#endif
 #ifdef HAVE_GETSPNAM
 # include <shadow.h>
 #endif
@@ -299,9 +305,44 @@ fail_delay( int retval ATTR_UNUSED, unsigned usec_delay ATTR_UNUSED,
 {}
 # endif
 
+# ifdef PAM_XDISPLAY
+/* Inspired by Eamon Walsh's patch for gdm. */
+static struct pam_xauth_data *
+getPAMXauthData( const char *xauth_file )
+{
+	FILE *fp;
+	Xauth *auth;
+	struct pam_xauth_data *ret;
+
+	if (!(fp = fopen( xauth_file, "r" )))
+		return NULL;
+
+	auth = XauReadAuth( fp );
+	fclose( fp );
+	if (!auth)
+		return NULL;
+
+	if ((ret = malloc( sizeof(*ret) + auth->name_length + 1 + auth->data_length ))) {
+		ret->name = (char *)ret + sizeof(*ret);
+		ret->namelen = auth->name_length;
+		memcpy( ret->name, auth->name, auth->name_length );
+		ret->name[ret->namelen] = 0;
+		ret->data = ret->name + ret->namelen + 1;
+		ret->datalen = auth->data_length;
+		memcpy( ret->data, auth->data, auth->data_length );
+	}
+
+	XauDisposeAuth( auth );
+	return ret;
+}
+# endif
+
 static int
 doPAMAuth( const char *psrv, struct pam_data *pdata )
 {
+# ifdef PAM_XDISPLAY
+	struct pam_xauth_data *pam_xauth;
+# endif
 	pam_gi_type pitem;
 	struct pam_conv pconv;
 	int pretc;
@@ -330,9 +371,23 @@ doPAMAuth( const char *psrv, struct pam_data *pdata )
 			goto pam_bail;
 	}
 # ifdef __sun__ /* Only Solaris <= 9, but checking it does not seem worth it. */
-	else if (pam_set_item( pamh, PAM_RHOST, 0 ) != PAM_SUCCESS)
+	else if ((pretc = pam_set_item( pamh, PAM_RHOST, 0 )) != PAM_SUCCESS)
 		goto pam_bail;
 # endif
+# ifdef PAM_XDISPLAY
+	if ((pretc = pam_set_item( pamh, PAM_XDISPLAY,
+	                           displayName( td ) )) != PAM_SUCCESS)
+		debug( "setting PAM_XDISPLAY failed: %s\n",
+		       pam_strerror( 0, pretc ) );
+	else if (td->authFile && (pam_xauth = getPAMXauthData( td->authFile ))) {
+		pretc = pam_set_item( pamh, PAM_XAUTHDATA, pam_xauth );
+		free( pam_xauth );
+		if (pretc != PAM_SUCCESS)
+			debug( "setting PAM_XAUTHDATA failed: %s\n",
+			       pam_strerror( 0, pretc ) );
+	}
+# endif
+	pam_set_item( pamh, PAM_USER_PROMPT, (void *)"Username:" );
 # ifdef PAM_FAIL_DELAY
 	pam_set_item( pamh, PAM_FAIL_DELAY, (void *)fail_delay );
 # endif
@@ -350,6 +405,14 @@ doPAMAuth( const char *psrv, struct pam_data *pdata )
 		pamh = 0;
 		V_RET;
 	}
+	/*
+	 * Do not even *think* about making that unconditional.
+	 * If a module thinks it needs to normalize user names (hello LDAP),
+	 * feed it with normalized data in the first place and/or do some
+	 * final normalization in startClient(). If that turns out impossible,
+	 * the module needs an own conversation plugin which does not cause
+	 * curuser being set.
+	 */
 	if (!curuser) {
 		debug( " asking PAM for user ...\n" );
 		pam_get_item( pamh, PAM_USER, &pitem );
@@ -1148,6 +1211,9 @@ static int removeAuth;
 static int removeSession;
 static int removeCreds;
 #endif
+#ifdef HAVE_CKCONNECTOR
+static CkConnector *ckConnector;
+#endif
 
 static GPipe ctlpipe;
 static GTalk ctltalk;
@@ -1196,6 +1262,17 @@ startClient( volatile int *pid )
 	char **theenv;
 	extern char **newenv; /* from libs.a, this is set up by setpenv */
 # endif
+#endif
+#ifdef HAVE_CKCONNECTOR
+	DBusError error;
+	int ckStatus;
+# ifdef XDMCP
+	char *remoteHostName = 0;
+	const char *spaceStr = "";
+# endif
+	char ckDeviceBuf[20] = "";
+	const char *ckDevice = ckDeviceBuf;
+	dbus_bool_t isLocal;
 #endif
 	char *failsafeArgv[2];
 	char *buf, *buf2;
@@ -1217,6 +1294,61 @@ startClient( volatile int *pid )
 
 	strcpy( curuser, p->pw_name ); /* Use normalized login name. */
 
+#ifdef HAVE_CKCONNECTOR
+	if (!(ckConnector = ck_connector_new())) {
+		logOutOfMem();
+		V_RET;
+	}
+	
+# ifdef HAVE_VTS
+	if (td->serverVT > 0)
+		sprintf( ckDeviceBuf, "/dev/tty%d", td->serverVT );
+# endif
+	isLocal = ((td->displayType & d_location) == dLocal);
+# ifdef XDMCP
+	if (!isLocal) {
+		int length, family;
+		CARD8 *data;
+		ARRAY8 addr;
+
+		family = convertAddr( (char *)td->peer.data, &length, &data );
+		addr.data = (unsigned char *)data;
+		addr.length = length;
+
+		/* We are not simply using the addr part of td->name as it might be
+		   numeric due to the SourceAddress settings */
+		remoteHostName = networkAddressToHostname(family, &addr);
+	}
+# endif
+
+	dbus_error_init( &error );
+	ckStatus = ck_connector_open_session_with_parameters( ckConnector, &error,
+			"unix-user", &p->pw_uid,
+			"x11-display-device", &ckDevice,
+			"x11-display", &td->name,
+			"is-local", &isLocal, /* meaning not entirely clear per doc */
+# ifdef XDMCP
+			"remote-host-name", remoteHostName ?
+			                       (const char **)&remoteHostName : &spaceStr,
+# endif
+			NULL );
+# ifdef XDMCP
+	if (remoteHostName)
+		free( remoteHostName );
+# endif
+	debug( "ck status: %d\n", ckStatus );
+	if (!ckStatus) {
+		if (dbus_error_is_set( &error )) {
+			logError( "Cannot open ConsoleKit session: %s\n", error.message );
+			dbus_error_free( &error );
+		} else {
+			logError( "Cannot open ConsoleKit session, likely OOM\n" );
+		}
+		ck_connector_unref( ckConnector );
+		V_RET;
+	}
+#endif
+
 #ifndef USE_PAM
 # ifdef _AIX
 	msg = NULL;
@@ -1267,6 +1399,10 @@ startClient( volatile int *pid )
 #if !defined(USE_PAM) && !defined(_AIX) && defined(KERBEROS)
 	if (krbtkfile[0] != '\0')
 		env = setEnv( env, "KRBTKFILE", krbtkfile );
+#endif
+#ifdef HAVE_CKCONNECTOR
+	env = setEnv( env, "XDG_SESSION_COOKIE",
+	                   ck_connector_get_cookie( ckConnector ) );
 #endif
 	userEnviron = inheritEnv( env, envvars );
 	env = systemEnv( 0, curuser );
@@ -1684,6 +1820,9 @@ clientExited( void )
 #ifdef USE_PAM
 	int pretc;
 #endif
+#ifdef HAVE_CKCONNECTOR
+	DBusError error;
+#endif
 
 	if (removeAuth) {
 		switch (source( systemEnviron, td->reset, td_setup )) {
@@ -1758,6 +1897,22 @@ clientExited( void )
 		pamh = 0;
 	}
 #endif
+
+#ifdef HAVE_CKCONNECTOR
+	if (ckConnector) {
+		dbus_error_init( &error );
+		if (!ck_connector_close_session( ckConnector, &error )) {
+			if (dbus_error_is_set( &error )) {
+				logError( "Cannot close ConsoleKit session: %s\n", error.message );
+				dbus_error_free( &error );
+			} else {
+				logError( "Cannot close ConsoleKit session, likely OOM\n" );
+			}
+		}
+		ck_connector_unref( ckConnector );
+		ckConnector = 0;
+	}
+#endif
 }
 
 void
