@@ -17,14 +17,13 @@
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */
-#ifndef _KLIPPERPOPUP_H_
-#define _KLIPPERPOPUP_H_
+#ifndef KLIPPERPOPUP_H
+#define KLIPPERPOPUP_H
+
+#include <QList>
+#include <QWidgetAction>
 
 #include <kmenu.h>
-#include <Qt3Support/Q3PtrList>
-
-//Added by qt3to4:
-#include <QKeyEvent>
 
 class History;
 class KHelpMenu;
@@ -75,13 +74,13 @@ private:
     /**
      * Contains the string shown if the menu is empty.
      */
-    QString QSempty;
+    QString m_qsEmpty;
 
     /**
      * Contains the string shown if the search string has no
      * matches and the menu is not empty.
      */
-    QString QSnomatch;
+    QString m_qsNoMatch;
 
     /**
      * The "document" (clipboard history)
@@ -91,12 +90,12 @@ private:
     /**
      * The help menu
      */
-    KHelpMenu* helpmenu;
+    KHelpMenu* m_helpmenu;
 
     /**
      * (unowned) actions to plug into the primary popup menu
      */
-    Q3PtrList<QAction> m_actions;
+    QList<QAction*> m_actions;
 
     /**
      * Proxy helper object used to track history items
@@ -109,14 +108,14 @@ private:
     KLineEdit* m_filterWidget;
 
     /**
-     * id of search widget, for convenience
+     * Action of search widget
      */
-    int m_filterWidgetId;
+    QWidgetAction *m_filterWidgetAction;
 
     /**
      * The current number of history items in the clipboard
      */
-    int n_history_items;
+    int m_nHistoryItems;
 
 Q_SIGNALS:
     void clearHistory();
