@@ -1,5 +1,6 @@
 /*
  *   Copyright (C) 2000 Matthias Elter <elter@kde.org>
+ *   Copyright (C) 2008 Laurent Montel <montel@kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -34,6 +35,7 @@
 #include <kdialog.h>
 #include <kkeysequencewidget.h>
 #include <klineedit.h>
+#include <klinespellchecking.h>
 #include <kmessagebox.h>
 #include <kicondialog.h>
 #include <kdesktopfile.h>
@@ -43,7 +45,9 @@
 #include <kshell.h>
 #include <khbox.h>
 
+#ifndef Q_WS_WIN
 #include "khotkeys.h"
+#endif
 
 #include "menuinfo.h"
 
@@ -51,18 +55,14 @@
 #include "basictab.moc"
 
 BasicTab::BasicTab( QWidget *parent )
-  : QWidget(parent)
+    : KTabWidget(parent)
 {
     _menuFolderInfo = 0;
     _menuEntryInfo = 0;
-
-    QGridLayout *layout = new QGridLayout(this );
-    layout->setMargin( KDialog::marginHint() );
-    layout->setSpacing( KDialog::spacingHint() );
-
+    
     // general group
-    QGroupBox *general_group = new QGroupBox(this);
-    QGridLayout *grid = new QGridLayout(general_group );
+    QWidget *general_group = new QWidget();
+    QGridLayout *grid = new QGridLayout(general_group);
     grid->setMargin( KDialog::marginHint() );
     grid->setSpacing( KDialog::spacingHint() );
 
@@ -71,27 +71,30 @@ BasicTab::BasicTab( QWidget *parent )
     // setup line inputs
     _nameEdit = new KLineEdit(general_group);
     _nameEdit->setAcceptDrops(false);
-    _descriptionEdit = new KLineEdit(general_group);
+    _descriptionEdit = new KLineSpellChecking(general_group);
     _descriptionEdit->setAcceptDrops(false);
-    _commentEdit = new KLineEdit(general_group);
+    _commentEdit = new KLineSpellChecking(general_group);
     _commentEdit->setAcceptDrops(false);
     _execEdit = new KUrlRequester(general_group);
     _execEdit->lineEdit()->setAcceptDrops(false);
     _execEdit->setWhatsThis(i18n(
-    "Following the command, you can have several place holders which will be replaced "
-    "with the actual values when the actual program is run:\n"
-    "%f - a single file name\n"
-    "%F - a list of files; use for applications that can open several local files at once\n"
-    "%u - a single URL\n"
-    "%U - a list of URLs\n"
-    "%d - the folder of the file to open\n"
-    "%D - a list of folders\n"
-    "%i - the icon\n"
-    "%m - the mini-icon\n"
-    "%c - the caption"));
+                                "Following the command, you can have several place holders which will be replaced "
+                                "with the actual values when the actual program is run:\n"
+                                "%f - a single file name\n"
+                                "%F - a list of files; use for applications that can open several local files at once\n"
+                                "%u - a single URL\n"
+                                "%U - a list of URLs\n"
+                                "%d - the folder of the file to open\n"
+                                "%D - a list of folders\n"
+                                "%i - the icon\n"
+                                "%m - the mini-icon\n"
+                                "%c - the caption"));
 
     _launchCB = new QCheckBox(i18n("Enable &launch feedback"), general_group);
     _systrayCB = new QCheckBox(i18n("&Place in system tray"), general_group);
+    _onlyShowInKdeCB = new QCheckBox( i18n( "Only show in KDE" ), general_group );
+    _hiddenEntryCB = new QCheckBox( i18n( "Hidden entry" ), general_group );
+    _hiddenEntryCB->hide();
 
     // setup labels
     _nameLabel = new QLabel(i18n("&Name:"),general_group);
@@ -120,7 +123,8 @@ BasicTab::BasicTab( QWidget *parent )
             SLOT(slotExecSelected()));
     connect(_launchCB, SIGNAL(clicked()), SLOT(launchcb_clicked()));
     connect(_systrayCB, SIGNAL(clicked()), SLOT(systraycb_clicked()));
-
+    connect(_onlyShowInKdeCB, SIGNAL( clicked() ), SLOT( onlyshowcb_clicked() ) );
+    connect( _hiddenEntryCB, SIGNAL( clicked() ), SLOT( hiddenentrycb_clicked() ) );
     // add line inputs to the grid
     grid->addWidget(_nameEdit, 0, 1, 1, 1);
     grid->addWidget(_descriptionEdit, 1, 1, 1, 1);
@@ -128,6 +132,8 @@ BasicTab::BasicTab( QWidget *parent )
     grid->addWidget(_execEdit, 3, 1, 1, 2);
     grid->addWidget(_launchCB, 4, 0, 1, 3 );
     grid->addWidget(_systrayCB, 5, 0, 1, 3 );
+    grid->addWidget(_onlyShowInKdeCB, 6, 0, 1, 3 );
+    grid->addWidget(_hiddenEntryCB, 7, 0, 1, 3 );
 
     // setup icon button
     _iconButton = new KIconButton(general_group);
@@ -135,25 +141,24 @@ BasicTab::BasicTab( QWidget *parent )
     _iconButton->setIconSize(48);
     connect(_iconButton, SIGNAL(iconChanged(QString)), SLOT(slotChanged()));
     grid->addWidget(_iconButton, 0, 2, 2, 1);
+    grid->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding), 8, 0, 1, 3);
 
     // add the general group to the main layout
-    layout->addWidget(general_group, 0, 0, 1, 2 );
+    addTab(general_group, i18n("General"));
+    
+    QWidget *advanced = new QWidget();
+    QVBoxLayout *advancedLayout = new QVBoxLayout(advanced);
 
     // path group
     _path_group = new QGroupBox(this);
-    QVBoxLayout *vbox = new QVBoxLayout(_path_group);
-    vbox->setMargin(KDialog::marginHint());
-    vbox->setSpacing(KDialog::spacingHint());
-
-    QWidget *hbox = new QWidget(_path_group);
-    QHBoxLayout *hboxLayout1 = new QHBoxLayout(hbox);
-    hbox->setLayout(hboxLayout1);
+    QHBoxLayout *hboxLayout1 = new QHBoxLayout(_path_group);
     hboxLayout1->setSpacing(KDialog::spacingHint());
+    hboxLayout1->setMargin(KDialog::marginHint());
 
-    _pathLabel = new QLabel(i18n("&Work path:"), hbox);
-	hboxLayout1->addWidget(_pathLabel);
-    _pathEdit = new KUrlRequester(hbox);
-	hboxLayout1->addWidget(_pathEdit);
+    _pathLabel = new QLabel(i18n("&Work path:"), _path_group);
+    hboxLayout1->addWidget(_pathLabel);
+    _pathEdit = new KUrlRequester(_path_group);
+    hboxLayout1->addWidget(_pathEdit);
     _pathEdit->setMode(KFile::Directory | KFile::LocalOnly);
     _pathEdit->lineEdit()->setAcceptDrops(false);
 
@@ -161,12 +166,11 @@ BasicTab::BasicTab( QWidget *parent )
 
     connect(_pathEdit, SIGNAL(textChanged(const QString&)),
             SLOT(slotChanged()));
-    vbox->addWidget(hbox);
-    layout->addWidget(_path_group, 1, 0, 1, 2 );
+    advancedLayout->addWidget(_path_group);
 
     // terminal group
     _term_group = new QGroupBox(this);
-    vbox = new QVBoxLayout(_term_group);
+    QVBoxLayout *vbox = new QVBoxLayout(_term_group);
     vbox->setMargin(KDialog::marginHint());
     vbox->setSpacing(KDialog::spacingHint());
 
@@ -174,21 +178,21 @@ BasicTab::BasicTab( QWidget *parent )
     connect(_terminalCB, SIGNAL(clicked()), SLOT(termcb_clicked()));
     vbox->addWidget(_terminalCB);
 
-    hbox = new QWidget(_term_group);
+    QWidget *hbox = new QWidget(_term_group);
     QHBoxLayout *hboxLayout2 = new QHBoxLayout(hbox);
     hbox->setLayout(hboxLayout2);
     hboxLayout2->setSpacing(KDialog::spacingHint());
     _termOptLabel = new QLabel(i18n("Terminal &options:"), hbox);
-	hboxLayout2->addWidget(_termOptLabel);
+    hboxLayout2->addWidget(_termOptLabel);
     _termOptEdit = new KLineEdit(hbox);
-	hboxLayout2->addWidget(_termOptEdit);
+    hboxLayout2->addWidget(_termOptEdit);
     _termOptEdit->setAcceptDrops(false);
     _termOptLabel->setBuddy(_termOptEdit);
 
     connect(_termOptEdit, SIGNAL(textChanged(const QString&)),
             SLOT(slotChanged()));
     vbox->addWidget(hbox);
-    layout->addWidget(_term_group, 2, 0, 1, 2 );
+    advancedLayout->addWidget(_term_group);
 
     _termOptEdit->setEnabled(false);
 
@@ -207,49 +211,42 @@ BasicTab::BasicTab( QWidget *parent )
     hbox->setLayout(hboxLayout3);
     hboxLayout3->setSpacing(KDialog::spacingHint());
     _uidLabel = new QLabel(i18n("&Username:"), hbox);
-	hboxLayout3->addWidget(_uidLabel);
+    hboxLayout3->addWidget(_uidLabel);
     _uidEdit = new KLineEdit(hbox);
-	hboxLayout3->addWidget(_uidEdit);
+    hboxLayout3->addWidget(_uidEdit);
     _uidEdit->setAcceptDrops(false);
     _uidLabel->setBuddy(_uidEdit);
 
     connect(_uidEdit, SIGNAL(textChanged(const QString&)),
 	    SLOT(slotChanged()));
     vbox->addWidget(hbox);
-    layout->addWidget(_uid_group, 3, 0, 1, 2 );
+    advancedLayout->addWidget(_uid_group);
 
     _uidEdit->setEnabled(false);
 
-    layout->setRowStretch(0, 2);
-
     // key binding group
     general_group_keybind = new QGroupBox(this);
-    layout->addWidget( general_group_keybind, 4, 0, 1, 2 );
-    // dummy widget in order to make it look a bit better
-    layout->addWidget( new QWidget(this), 5, 0 );
-    layout->setRowStretch( 5, 4 );
-    QGridLayout *grid_keybind = new QGridLayout(general_group_keybind );
-    grid_keybind->setMargin( KDialog::marginHint() );
-    grid_keybind->setSpacing( KDialog::spacingHint());
+    QHBoxLayout *keybindLayout = new QHBoxLayout(general_group_keybind);
+    keybindLayout->setMargin( KDialog::marginHint() );
+    keybindLayout->setSpacing( KDialog::spacingHint());
 
-    //_keyEdit = new KLineEdit(general_group_keybind);
-    //_keyEdit->setReadOnly( true );
-    //_keyEdit->setText( "" );
-    //QPushButton* _keyButton = new QPushButton( i18n( "Change" ),
-    //                                           general_group_keybind );
-    //connect( _keyButton, SIGNAL( clicked()), this, SLOT( keyButtonPressed()));
     _keyEdit = new KKeySequenceWidget(general_group_keybind);
+    _keyEdit->setMultiKeyShortcutsAllowed(false);
     QLabel *l = new QLabel( i18n("Current shortcut &key:"), general_group_keybind);
     l->setBuddy( _keyEdit );
-    grid_keybind->addWidget(l, 0, 0);
+    keybindLayout->addWidget(l);
     connect( _keyEdit, SIGNAL(keySequenceChanged(const QKeySequence&)),
              this, SLOT(slotCapturedKeySequence(const QKeySequence&)));
-    grid_keybind->addWidget(_keyEdit, 0, 1);
-    //grid_keybind->addWidget(_keyButton, 0, 2 );
-
+    keybindLayout->addWidget(_keyEdit);
+    advancedLayout->addWidget( general_group_keybind );
+    
+    advancedLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding));
+    
+    addTab(advanced, i18n("Advanced"));
+#ifndef Q_WS_WIN
     if (!KHotKeys::present())
-       general_group_keybind->hide();
-
+        general_group_keybind->hide();
+#endif
     slotDisableAction();
 }
 
@@ -263,6 +260,8 @@ void BasicTab::slotDisableAction()
     _execEdit->setEnabled(false);
     _launchCB->setEnabled(false);
     _systrayCB->setEnabled(false);
+    _onlyShowInKdeCB->setEnabled( false );
+    _hiddenEntryCB->setEnabled( false );
     _nameLabel->setEnabled(false);
     _descriptionLabel->setEnabled(false);
     _commentLabel->setEnabled(false);
@@ -285,6 +284,8 @@ void BasicTab::enableWidgets(bool isDF, bool isDeleted)
     _execEdit->setEnabled(isDF && !isDeleted);
     _launchCB->setEnabled(isDF && !isDeleted);
     _systrayCB->setEnabled(isDF && !isDeleted);
+    _onlyShowInKdeCB->setEnabled( isDF && !isDeleted );
+    _hiddenEntryCB->setEnabled( isDF && !isDeleted );
     _nameLabel->setEnabled(!isDeleted);
     _descriptionLabel->setEnabled(!isDeleted);
     _commentLabel->setEnabled(!isDeleted);
@@ -316,13 +317,15 @@ void BasicTab::setFolderInfo(MenuFolderInfo *folderInfo)
     _iconButton->setIcon(folderInfo->icon);
 
     // clean all disabled fields and return
-    _execEdit->lineEdit()->setText("");
-    _pathEdit->lineEdit()->setText("");
-    _termOptEdit->setText("");
-    _uidEdit->setText("");
+    _execEdit->lineEdit()->clear();
+    _pathEdit->lineEdit()->clear();
+    _termOptEdit->clear();
+    _uidEdit->clear();
     _launchCB->setChecked(false);
     _systrayCB->setChecked(false);
     _terminalCB->setChecked(false);
+    _onlyShowInKdeCB->setChecked( false );
+    _hiddenEntryCB->setChecked( false );
     _uidCB->setChecked(false);
     _keyEdit->clearKeySequence();
 
@@ -338,19 +341,22 @@ void BasicTab::setEntryInfo(MenuEntryInfo *entryInfo)
 
     if (!entryInfo)
     {
-       _nameEdit->setText(QString());
-       _descriptionEdit->setText(QString());
-       _commentEdit->setText(QString());
-       _iconButton->setIcon(QString());
+       _nameEdit->clear();
+       _descriptionEdit->clear();
+       _commentEdit->clear();
+       _iconButton->setIcon( QString() );
 
        // key binding part
        _keyEdit->clearKeySequence();
-       _execEdit->lineEdit()->setText(QString());
-       _systrayCB->setChecked(false);
 
-       _pathEdit->lineEdit()->setText(QString());
-       _termOptEdit->setText(QString());
-       _uidEdit->setText(QString());
+       _execEdit->lineEdit()->clear();
+       _systrayCB->setChecked(false);
+       _onlyShowInKdeCB->setChecked( false );
+       _hiddenEntryCB->setChecked( false );
+
+       _pathEdit->lineEdit()->clear();
+       _termOptEdit->clear();
+       _uidEdit->clear();
 
        _launchCB->setChecked(false);
        _terminalCB->setChecked(false);
@@ -370,6 +376,7 @@ void BasicTab::setEntryInfo(MenuEntryInfo *entryInfo)
     _iconButton->setIcon(df->readIcon());
 
     // key binding part
+#ifndef Q_WS_WIN
     if( KHotKeys::present())
     {
         if ( !entryInfo->shortcut().isEmpty() )
@@ -377,9 +384,9 @@ void BasicTab::setEntryInfo(MenuEntryInfo *entryInfo)
         else
             _keyEdit->clearKeySequence();
     }
-
-    QString temp = df->desktopGroup().readPathEntry("Exec", QString());
-    if (temp.left(12) == "ksystraycmd ")
+#endif
+    QString temp = df->desktopGroup().readEntry("Exec");
+    if (temp.startsWith("ksystraycmd "))
     {
       _execEdit->lineEdit()->setText(temp.right(temp.length()-12));
       _systrayCB->setChecked(true);
@@ -399,6 +406,18 @@ void BasicTab::setEntryInfo(MenuEntryInfo *entryInfo)
     else // backwards comp.
         _launchCB->setChecked(df->desktopGroup().readEntry("X-KDE-StartupNotify", true));
 
+    _onlyShowInKdeCB->setChecked( false );
+    if ( df->desktopGroup().hasKey( "OnlyShowIn") )
+    {
+        if ( df->desktopGroup().readXdgListEntry("OnlyShowIn").contains( "KDE" ) )
+            _onlyShowInKdeCB->setChecked( true );
+
+    }
+    if ( df->desktopGroup().hasKey( "NoDisplay" ) )
+        _hiddenEntryCB->setChecked( df->desktopGroup().readEntry( "NoDisplay", true ) );
+    else
+        _hiddenEntryCB->setChecked( false );
+
     if(df->desktopGroup().readEntry("Terminal", 0) == 1)
         _terminalCB->setChecked(true);
     else
@@ -423,9 +442,9 @@ void BasicTab::apply()
         KConfigGroup dg = df->desktopGroup();
         dg.writeEntry("Comment", _commentEdit->text());
         if (_systrayCB->isChecked())
-          dg.writePathEntry("Exec", _execEdit->lineEdit()->text().prepend("ksystraycmd "));
+          dg.writeEntry("Exec", _execEdit->lineEdit()->text().prepend("ksystraycmd "));
         else
-          dg.writePathEntry("Exec", _execEdit->lineEdit()->text());
+          dg.writeEntry("Exec", _execEdit->lineEdit()->text());
 
         dg.writePathEntry("Path", _pathEdit->lineEdit()->text());
 
@@ -438,6 +457,9 @@ void BasicTab::apply()
         dg.writeEntry("X-KDE-SubstituteUID", _uidCB->isChecked());
         dg.writeEntry("X-KDE-Username", _uidEdit->text());
         dg.writeEntry("StartupNotify", _launchCB->isChecked());
+        dg.writeEntry( "NoDisplay", _hiddenEntryCB->isChecked() );
+        if ( _onlyShowInKdeCB->isChecked() )
+            dg.writeXdgListEntry( "OnlyShowIn", QStringList()<<"KDE" );
     }
     else
     {
@@ -469,6 +491,16 @@ void BasicTab::systraycb_clicked()
     slotChanged();
 }
 
+void BasicTab::onlyshowcb_clicked()
+{
+    slotChanged();
+}
+
+void BasicTab::hiddenentrycb_clicked()
+{
+    slotChanged();
+}
+
 void BasicTab::termcb_clicked()
 {
     _termOptEdit->setEnabled(_terminalCB->isChecked());
@@ -495,38 +527,26 @@ void BasicTab::slotCapturedKeySequence(const QKeySequence& seq)
     if (signalsBlocked())
        return;
     KShortcut cut(seq, QKeySequence());
-
-#ifdef __GNUC__
-#warning the following lines can be implemented again using the new functions in KGlobalAccel
-#endif
-    /*if( KKeyChooser::checkGlobalShortcutsConflict( cut, true, topLevelWidget())
-        || KKeyChooser::checkStandardShortcutsConflict( cut, true, topLevelWidget()))
-        return;*/
-
-    if ( KHotKeys::present() )
+#ifndef Q_WS_WIN
+    if (_menuEntryInfo->isShortcutAvailable( cut ) && KHotKeys::present() )
     {
-       if (!_menuEntryInfo->isShortcutAvailable( cut ) )
-       {
-          KService::Ptr service;
-          emit findServiceShortcut(cut, service);
-          if (!service)
-             service = KHotKeys::findMenuEntry(cut.toString());
-          if (service)
-          {
-             KMessageBox::sorry(this, i18n("<qt>The key <b>%1</b> can not be used here because it is already used to activate <b>%2</b>.</qt>", cut.toString(), service->name()));
-             _keyEdit->clearKeySequence();
-             return;
-          }
-          else
-          {
-             KMessageBox::sorry(this, i18n("<qt>The key <b>%1</b> can not be used here because it is already in use.</qt>", cut.toString()));
-             _keyEdit->clearKeySequence();
-             return;
-          }
-       }
        _menuEntryInfo->setShortcut( cut );
     }
+    else
+    {
+       // We will not assign the shortcut so reset the visible key sequence
+       _keyEdit->setKeySequence(QKeySequence());
+    }
+#endif
     if (_menuEntryInfo)
        emit changed( _menuEntryInfo );
 }
 
+
+void BasicTab::updateHiddenEntry( bool _hidden )
+{
+    if ( _hidden )
+        _hiddenEntryCB->show();
+    else
+        _hiddenEntryCB->hide();
+}
