@@ -24,26 +24,59 @@
 
 namespace Plasma
 {
+    class RunnerManager;
+    class FrameSvg;
     class Svg;
 }
 
+class KRunnerConfigDialog;
+
 class KRunnerDialog : public KDialog
 {
     Q_OBJECT
 
     public:
-        explicit KRunnerDialog( QWidget * parent = 0,
-                                Qt::WindowFlags f =  Qt::Dialog | Qt::FramelessWindowHint );
+        explicit KRunnerDialog(Plasma::RunnerManager *manager, QWidget *parent = 0,
+                               Qt::WindowFlags f =  Qt::Dialog | Qt::FramelessWindowHint);
         virtual ~KRunnerDialog();
 
+    public Q_SLOTS:
+        virtual void display(const QString& term = QString()) = 0;
+        virtual void clearHistory() = 0;
+
+        void switchUser();
+
     protected:
-        void paintEvent( QPaintEvent *e );
-        void resizeEvent( QResizeEvent *e );
-        void mousePressEvent( QMouseEvent *e );
+        void paintEvent(QPaintEvent *event);
+        void resizeEvent(QResizeEvent *event);
+        void mousePressEvent(QMouseEvent *event);
+        bool event(QEvent *event);
+
+        virtual void setStaticQueryMode(bool staticQuery);
+
+    protected Q_SLOTS:
+        void showConfigDialog();
+
+    private Q_SLOTS:
+        /**
+         * React to theme changes
+         */
+        void themeUpdated();
+
+        /**
+         * React to configuration changes
+         */
+        void configCompleted();
+
+    protected:
+        Plasma::Svg *m_iconSvg;
+        Plasma::RunnerManager *m_runnerManager;
 
     private:
         void paintBackground(QPainter* painter, const QRect &exposedRect);
-        Plasma::Svg *m_background;
+
+        KRunnerConfigDialog *m_configDialog;
+        Plasma::FrameSvg *m_background;
         QPixmap *m_cachedBackground;
 };
 
