@@ -4,8 +4,9 @@
     Copyright (c) 1999, 2000 Chris Schlaeger <cs@kde.org>
 
     This program is free software; you can redistribute it and/or
-    modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    modify it under the terms of the GNU General Public
+    License version 2 or at your option version 3 as published by
+    the Free Software Foundation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,6 +28,7 @@
 #include <QtCore/QHash>
 #include <QtCore/QStringList>
 #include <QtCore/QObject>
+#include <QtCore/QPointer>
 
 #include "SensorAgent.h"
 
@@ -58,7 +60,7 @@ class KDE_EXPORT SensorManager : public QObject
         QString mMessage;
     };
 
-    explicit SensorManager();
+    explicit SensorManager(QObject * parent = 0);
     ~SensorManager();
 
     bool engage( const QString &hostName, const QString &shell = "ssh",
@@ -66,17 +68,13 @@ class KDE_EXPORT SensorManager : public QObject
     /* Returns true if we are connected or trying to connect to the host given
      */
     bool isConnected( const QString &hostName );
-    void requestDisengage( const SensorAgent *agent );
-    bool disengage( const SensorAgent *agent );
+    bool disengage( SensorAgent *agent );
     bool disengage( const QString &hostName );
     bool resynchronize( const QString &hostName );
-    void hostLost( const SensorAgent *agent );
     void notify( const QString &msg ) const;
 
     void setBroadcaster( QWidget *wdg );
 
-    virtual bool event( QEvent *event );
-
     bool sendRequest( const QString &hostName, const QString &request,
                       SensorClient *client, int id = 0 );
 
@@ -93,12 +91,15 @@ class KDE_EXPORT SensorManager : public QObject
     void saveProperties( KConfigGroup& cfg );
 
     void disconnectClient( SensorClient *client );
+    /** Call to retranslate all the strings - for example if the language has changed */
+    void retranslate();
 
   public Q_SLOTS:
     void reconfigure( const SensorAgent *agent );
 
   Q_SIGNALS:
     void update();
+    void hostAdded(KSGRD::SensorAgent *sensorAgent, const QString &hostName);
     void hostConnectionLost( const QString &hostName );
 
   protected:
@@ -118,7 +119,7 @@ class KDE_EXPORT SensorManager : public QObject
     QStringList mHostList;
     QStringList mCommandList;
 
-    QWidget* mBroadcaster;
+    QPointer<QWidget> mBroadcaster;
 };
 
 KDE_EXPORT extern SensorManager* SensorMgr;
