@@ -116,7 +116,7 @@ namespace KWin
        Since at this time it's known that the new window doesn't belong
        to any existing application (better said, the application doesn't
        have any other window mapped), it is either the very first window
-       of the application, or its the only window of the application
+       of the application, or it is the only window of the application
        that was hidden before. The latter case is handled by removing
        the property from windows before withdrawing them, making
        the timestamp empty for next mapping of the window. In the sooner
@@ -416,16 +416,16 @@ bool Workspace::activateNextClient( Client* c )
     if( focusChangeEnabled())
         {
         if ( options->focusPolicyIsReasonable())
-            { // search the focus_chain for a client to transfer focus to
-              // if 'c' is transient, transfer focus to the first suitable mainwindow
+            { // search the focus_chain for a client to transfer focus to,
+              // first try to transfer focus to the first suitable window in the group
             Client* get_focus = NULL;
-            const ClientList mainwindows = ( c != NULL ? c->mainClients() : ClientList());
+            const ClientList windows = ( c != NULL ? c->group()->members() : ClientList());
 	    for ( int i = focus_chain[ currentDesktop() ].size() - 1;
                   i >= 0;
                   --i )
                 {
                 Client* ci = focus_chain[ currentDesktop() ].at( i );
-                if( !ci->isShown( false )
+                if( c == ci || !ci->isShown( false )
                     || !ci->isOnCurrentDesktop())
                     continue;
                 if( options->separateScreenFocus )
@@ -435,7 +435,7 @@ bool Workspace::activateNextClient( Client* c )
                     if( c == NULL && !ci->isOnScreen( activeScreen()))
                         continue;
                     }
-                if( mainwindows.contains( ci ))
+                if( windows.contains( ci ))
                     {
                     get_focus = ci;
                     break;
@@ -861,6 +861,8 @@ void Client::setActive( bool act )
     if ( active == act )
         return;
     active = act;
+    const int ruledOpacity = active ? rules()->checkOpacityActive(opacity()*100) : rules()->checkOpacityInactive(opacity()*100);
+    setOpacity( ruledOpacity/100.0 );
     workspace()->setActiveClient( act ? this : NULL, Allowed );
     
     if ( active )
@@ -875,8 +877,8 @@ void Client::setActive( bool act )
     StackingUpdatesBlocker blocker( workspace());
     workspace()->updateClientLayer( this ); // active windows may get different layer
     ClientList mainclients = mainClients();
-    for( ClientList::ConstIterator it = mainclients.begin();
-         it != mainclients.end();
+    for( ClientList::ConstIterator it = mainclients.constBegin();
+         it != mainclients.constEnd();
          ++it )
         if( (*it)->isFullScreen()) // fullscreens go high even if their transient is active
             workspace()->updateClientLayer( *it );
@@ -884,6 +886,7 @@ void Client::setActive( bool act )
         decoration->activeChange();
     updateMouseGrab();
     updateUrgency(); // demand attention again if it's still urgent
+    workspace()->checkUnredirect();
     }
 
 void Client::startupIdChanged()
