@@ -20,6 +20,7 @@
   Boston, MA 02110-1301, USA.
  */
 
+#include "plastikclient.h"
 #include <klocale.h>
 
 #include <QBitmap>
@@ -30,9 +31,8 @@
 #include <QLayout>
 #include <QPainter>
 #include <QPixmap>
-#include <qdesktopwidget.h>
+#include <QDesktopWidget>
 
-#include "plastikclient.h"
 #include "plastikbutton.h"
 
 #include <KColorScheme>
