@@ -104,7 +104,7 @@ bool checkGLError( const char* txt )
     GLenum err = glGetError();
     if( err != GL_NO_ERROR )
         {
-        kWarning() << "GL error (" << txt << "): 0x" << QString::number( err, 16 ) ;
+        kWarning(1212) << "GL error (" << txt << "): 0x" << QString::number( err, 16 ) ;
         return true;
         }
     return false;
@@ -128,18 +128,10 @@ int nearestPowerOfTwo( int x )
 void renderGLGeometry( int count, const float* vertices, const float* texture, const float* color,
     int dim, int stride )
     {
-    return renderGLGeometry( false, QRegion(), count, vertices, texture, color, dim, stride );
+    return renderGLGeometry( infiniteRegion(), count, vertices, texture, color, dim, stride );
     }
 
-void renderGLGeometry( int mask, const QRegion& region, int count,
-    const float* vertices, const float* texture, const float* color,
-    int dim, int stride )
-    {
-    return renderGLGeometry( !( mask & ( Effect::PAINT_WINDOW_TRANSFORMED | Effect::PAINT_SCREEN_TRANSFORMED )),
-        region, count, vertices, texture, color, dim, stride );
-    }
-
-void renderGLGeometry( bool clip, const QRegion& region, int count,
+void renderGLGeometry( const QRegion& region, int count,
     const float* vertices, const float* texture, const float* color,
     int dim, int stride )
     {
@@ -166,40 +158,17 @@ void renderGLGeometry( bool clip, const QRegion& region, int count,
             }
         }
 
-    // Render
-    if( !clip )
+    // Clip using scissoring
+    PaintClipper pc( region );
+    for( PaintClipper::Iterator iterator;
+         !iterator.isDone();
+         iterator.next())
         {
-        // Just draw the entire window, no clipping
         if( use_arrays )
             glDrawArrays( GL_QUADS, 0, count );
         else
             renderGLGeometryImmediate( count, vertices, texture, color, dim, stride );
         }
-    else
-        {
-        // Make sure there's only a single quad (no transformed vertices)
-        // Clip using scissoring
-        glEnable( GL_SCISSOR_TEST );
-        int dh = displayHeight();
-        if( use_arrays )
-            {
-            foreach( QRect r, region.rects())
-                {
-                // Scissor rect has to be given in OpenGL coords
-                glScissor(r.x(), dh - r.y() - r.height(), r.width(), r.height());
-                glDrawArrays( GL_QUADS, 0, count );
-                }
-            }
-        else
-            {
-            foreach( QRect r, region.rects())
-                {
-                // Scissor rect has to be given in OpenGL coords
-                glScissor(r.x(), dh - r.y() - r.height(), r.width(), r.height());
-                renderGLGeometryImmediate( count, vertices, texture, color, dim, stride );
-                }
-            }
-        }
 
     if( use_arrays )
         {
@@ -329,7 +298,9 @@ void renderRoundBox( const QRect& area, float roundness, GLTexture* texture )
     glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
     // We have two elements per vertex in the verts array
     int verticesCount = verts.count() / 2;
+    texture->enableNormalizedTexCoords();
     renderGLGeometry( verticesCount, verts.data(), texcoords.data() );
+    texture->disableNormalizedTexCoords();
     texture->unbind();
 
     glPopMatrix();
@@ -399,6 +370,8 @@ GLTexture::GLTexture( int width, int height )
 GLTexture::~GLTexture()
     {
     discard();
+    assert( mUnnormalizeActive == 0 );
+    assert( mNormalizeActive == 0 );
     }
 
 void GLTexture::init()
@@ -409,6 +382,8 @@ void GLTexture::init()
     y_inverted = false;
     can_use_mipmaps = false;
     has_valid_mipmaps = false;
+    mUnnormalizeActive = 0;
+    mNormalizeActive = 0;
     }
 
 void GLTexture::initStatic()
@@ -505,13 +480,7 @@ void GLTexture::unbind()
     glDisable( mTarget );
     }
 
-void GLTexture::render( int mask, QRegion region, const QRect& rect )
-    {
-    return render( !( mask & ( Effect::PAINT_WINDOW_TRANSFORMED | Effect::PAINT_SCREEN_TRANSFORMED )),
-        region, rect );
-    }
-
-void GLTexture::render( bool clip, QRegion region, const QRect& rect )
+void GLTexture::render( QRegion region, const QRect& rect )
     {
     const float verts[ 4 * 2 ] =
         {
@@ -522,16 +491,21 @@ void GLTexture::render( bool clip, QRegion region, const QRect& rect )
         };
     const float texcoords[ 4 * 2 ] =
         {
-        0, 1,
+        0, 1, // y needs to be swapped (normalized coords)
         0, 0,
         1, 0,
         1, 1
         };
-    renderGLGeometry( clip, region, 4, verts, texcoords );
+    enableNormalizedTexCoords();
+    renderGLGeometry( region, 4, verts, texcoords );
+    disableNormalizedTexCoords();
     }
 
 void GLTexture::enableUnnormalizedTexCoords()
     {
+    assert( mNormalizeActive == 0 );
+    if( mUnnormalizeActive++ != 0 )
+        return;
     // update texture matrix to handle GL_TEXTURE_2D and GL_TEXTURE_RECTANGLE
     glMatrixMode( GL_TEXTURE );
     glPushMatrix();
@@ -549,6 +523,38 @@ void GLTexture::enableUnnormalizedTexCoords()
 
 void GLTexture::disableUnnormalizedTexCoords()
     {
+    if( --mUnnormalizeActive != 0 )
+        return;
+    // Restore texture matrix
+    glMatrixMode( GL_TEXTURE );
+    glPopMatrix();
+    glMatrixMode( GL_MODELVIEW );
+    }
+
+void GLTexture::enableNormalizedTexCoords()
+    {
+    assert( mUnnormalizeActive == 0 );
+    if( mNormalizeActive++ != 0 )
+        return;
+    // update texture matrix to handle GL_TEXTURE_2D and GL_TEXTURE_RECTANGLE
+    glMatrixMode( GL_TEXTURE );
+    glPushMatrix();
+    glLoadIdentity();
+    glScalef( mSize.width() * mScale.width(), mSize.height() * mScale.height(), 1 );
+    if( y_inverted )
+        {
+        // Modify texture matrix so that we could always use non-opengl
+        //  coordinates for textures
+        glScalef( 1, -1, 1 );
+        glTranslatef( 0, -1, 0 );
+        }
+    glMatrixMode( GL_MODELVIEW );
+    }
+
+void GLTexture::disableNormalizedTexCoords()
+    {
+    if( --mNormalizeActive != 0 )
+        return;
     // Restore texture matrix
     glMatrixMode( GL_TEXTURE );
     glPopMatrix();
@@ -641,27 +647,69 @@ void GLTexture::enableFilter()
         }
     }
 
-QImage GLTexture::convertToGLFormat( const QImage& img ) const
-    {
-    // This method has been copied from Qt's QGLWidget::convertToGLFormat()
-    QImage res = img.convertToFormat(QImage::Format_ARGB32);
-    res = res.mirrored();
+static void convertToGLFormatHelper(QImage &dst, const QImage &img, GLenum texture_format)
+    { // Copied from Qt
+    Q_ASSERT(dst.size() == img.size());
+    Q_ASSERT(dst.depth() == 32);
+    Q_ASSERT(img.depth() == 32);
 
-    if (QSysInfo::ByteOrder == QSysInfo::BigEndian) {
-        // Qt has ARGB; OpenGL wants RGBA
-        for (int i=0; i < res.height(); i++) {
-            uint *p = (uint*)res.scanLine(i);
-            uint *end = p + res.width();
-            while (p < end) {
-                *p = (*p << 8) | ((*p >> 24) & 0xFF);
-                p++;
+    const int width = img.width();
+    const int height = img.height();
+    const uint *p = (const uint*) img.scanLine(img.height() - 1);
+    uint *q = (uint*) dst.scanLine(0);
+
+    if (texture_format == GL_BGRA) {
+        if (QSysInfo::ByteOrder == QSysInfo::BigEndian) {
+            // mirror + swizzle
+            for (int i=0; i < height; ++i) {
+                const uint *end = p + width;
+                while (p < end) {
+                    *q = ((*p << 24) & 0xff000000)
+                         | ((*p >> 24) & 0x000000ff)
+                         | ((*p << 8) & 0x00ff0000)
+                         | ((*p >> 8) & 0x0000ff00);
+                    p++;
+                    q++;
+                }
+                p -= 2 * width;
+            }
+        } else {
+            const uint bytesPerLine = img.bytesPerLine();
+            for (int i=0; i < height; ++i) {
+                memcpy(q, p, bytesPerLine);
+                q += width;
+                p -= width;
+            }
+        }
+    } else {
+        if (QSysInfo::ByteOrder == QSysInfo::BigEndian) {
+            for (int i=0; i < height; ++i) {
+                const uint *end = p + width;
+                while (p < end) {
+                    *q = (*p << 8) | ((*p >> 24) & 0xFF);
+                    p++;
+                    q++;
+                }
+                p -= 2 * width;
+            }
+        } else {
+            for (int i=0; i < height; ++i) {
+                const uint *end = p + width;
+                while (p < end) {
+                    *q = ((*p << 16) & 0xff0000) | ((*p >> 16) & 0xff) | (*p & 0xff00ff00);
+                    p++;
+                    q++;
+                }
+                p -= 2 * width;
             }
         }
     }
-    else {
-        // Qt has ARGB; OpenGL wants ABGR (i.e. RGBA backwards)
-        res = res.rgbSwapped();
-    }
+}
+
+QImage GLTexture::convertToGLFormat( const QImage& img ) const
+    { // Copied from Qt's QGLWidget::convertToGLFormat()
+    QImage res(img.size(), QImage::Format_ARGB32);
+    convertToGLFormatHelper(res, img.convertToFormat(QImage::Format_ARGB32), GL_RGBA);
     return res;
     }
 
@@ -686,6 +734,8 @@ GLShader::GLShader(const QString& vertexfile, const QString& fragmentfile)
     mValid = false;
     mVariableLocations = 0;
     mProgram = 0;
+    mTextureWidth = -1.0f;
+    mTextureHeight = -1.0f;
 
     loadFromFiles(vertexfile, fragmentfile);
     }
@@ -902,6 +952,25 @@ bool GLShader::setAttribute(const QString& name, float value)
     return (location >= 0);
     }
 
+void GLShader::setTextureHeight(float height)
+    {
+    mTextureHeight = height;
+    }
+
+void GLShader::setTextureWidth(float width)
+    {
+    mTextureWidth = width;
+    }
+
+float GLShader::textureHeight()
+    {
+    return mTextureHeight;
+    }
+
+float GLShader::textureWidth()
+    {
+    return mTextureWidth;
+    }
 
 
 /***  GLRenderTarget  ***/
