@@ -1,4 +1,4 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
@@ -6,9 +6,19 @@ Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 1997 to 2002 Cristian Tibirna <tibirna@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "placement.h"
 
@@ -177,7 +187,7 @@ void Placement::placeSmart(Client* c, const QRect& area, Policy /*next*/ )
             cxl = x; cxr = x + cw;
             cyt = y; cyb = y + ch;
             ClientList::ConstIterator l;
-            for(l = m_WorkspacePtr->stackingOrder().begin(); l != m_WorkspacePtr->stackingOrder().end() ; ++l) 
+            for(l = m_WorkspacePtr->stackingOrder().constBegin(); l != m_WorkspacePtr->stackingOrder().constEnd() ; ++l) 
                 {
                 if((*l)->isOnDesktop(desktop) &&
                    (*l)->isShown( false ) && (*l) != c) 
@@ -233,7 +243,7 @@ void Placement::placeSmart(Client* c, const QRect& area, Policy /*next*/ )
 
             // compare to the position of each client on the same desk
             ClientList::ConstIterator l;
-            for(l = m_WorkspacePtr->stackingOrder().begin(); l != m_WorkspacePtr->stackingOrder().end() ; ++l) 
+            for(l = m_WorkspacePtr->stackingOrder().constBegin(); l != m_WorkspacePtr->stackingOrder().constEnd() ; ++l) 
                 {
 
                 if ((*l)->isOnDesktop(desktop) &&
@@ -268,7 +278,7 @@ void Placement::placeSmart(Client* c, const QRect& area, Policy /*next*/ )
 
             //test the position of each window on the desk
             ClientList::ConstIterator l;
-            for(l = m_WorkspacePtr->stackingOrder().begin(); l != m_WorkspacePtr->stackingOrder().end() ; ++l) 
+            for(l = m_WorkspacePtr->stackingOrder().constBegin(); l != m_WorkspacePtr->stackingOrder().constEnd() ; ++l) 
                 {
                 if((*l)->isOnDesktop(desktop) &&
                     (*l) != c   &&  c->isShown( false )) 
@@ -364,12 +374,14 @@ void Placement::placeCascaded (Client* c, QRect& area, Policy nextPlacement)
     if ((yp + ch) > H) yp = Y;
 
     if ((xp + cw) > W)
+        {
         if (!yp) 
         {
         place(c,area,nextPlacement);
         return;
         }
-    else xp = X;
+        else xp = X;
+        }
 
   //if this isn't the first window
     if (cci[dn].pos.x() != X && cci[dn].pos.y() != Y) 
@@ -471,8 +483,8 @@ void Placement::placeOnMainWindow(Client* c, QRect& area, Policy nextPlacement )
     Client* place_on = NULL;
     Client* place_on2 = NULL;
     int mains_count = 0;
-    for( ClientList::ConstIterator it = mainwindows.begin();
-         it != mainwindows.end();
+    for( ClientList::ConstIterator it = mainwindows.constBegin();
+         it != mainwindows.constEnd();
          ++it )
         {
         if( mainwindows.count() > 1 && (*it)->isSpecialWindow())
@@ -717,8 +729,8 @@ int Workspace::packPositionLeft( const Client* cl, int oldx, bool left_edge ) co
             QPoint( cl->geometry().left() - 1, cl->geometry().center().y()), cl->desktop()).left();
     if( oldx <= newx )
         return oldx;
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it)
         {
         if( !(*it)->isShown( false ) || !(*it)->isOnDesktop( active_client->desktop()))
@@ -740,8 +752,8 @@ int Workspace::packPositionRight( const Client* cl, int oldx, bool right_edge )
             QPoint( cl->geometry().right() + 1, cl->geometry().center().y()), cl->desktop()).right();
     if( oldx >= newx )
         return oldx;
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it)
         {
         if( !(*it)->isShown( false ) || !(*it)->isOnDesktop( cl->desktop()))
@@ -763,8 +775,8 @@ int Workspace::packPositionUp( const Client* cl, int oldy, bool top_edge ) const
             QPoint( cl->geometry().center().x(), cl->geometry().top() - 1 ), cl->desktop()).top();
     if( oldy <= newy )
         return oldy;
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it)
         {
         if( !(*it)->isShown( false ) || !(*it)->isOnDesktop( cl->desktop()))
@@ -786,8 +798,8 @@ int Workspace::packPositionDown( const Client* cl, int oldy, bool bottom_edge )
             QPoint( cl->geometry().center().x(), cl->geometry().bottom() + 1 ), cl->desktop()).bottom();
     if( oldy >= newy )
         return oldy;
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it)
         {
         if( !(*it)->isShown( false ) || !(*it)->isOnDesktop( cl->desktop()))
