@@ -77,6 +77,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "deleted.h"
 #include "effects.h"
 
+#include <kephal/screens.h>
+
 namespace KWin
 {
 
@@ -87,8 +89,9 @@ namespace KWin
 Scene* scene;
 
 Scene::Scene( Workspace* ws )
-    : wspace( ws ),
-    has_waitSync( false )
+    : wspace( ws )
+    , has_waitSync( false )
+    , selfCheckDone( false )
     {
     }
     
@@ -116,7 +119,7 @@ void Scene::paintScreen( int* mask, QRegion* region )
         *mask &= ~PAINT_SCREEN_REGION;
         *region = infiniteRegion();
         }
-    else if(  *mask & PAINT_SCREEN_REGION )
+    else if( *mask & PAINT_SCREEN_REGION )
         { // make sure not to go outside visible screen
         *region &= QRegion( 0, 0, displayWidth(), displayHeight());
         }
@@ -135,6 +138,8 @@ void Scene::paintScreen( int* mask, QRegion* region )
     *region |= painted_region;
     // make sure not to go outside of the screen area
     *region &= QRegion( 0, 0, displayWidth(), displayHeight());
+    // make sure all clipping is restored
+    Q_ASSERT( !PaintClipper::clip());
     }
 
 // Compute time since the last painting pass.
@@ -189,16 +194,19 @@ void Scene::paintGenericScreen( int orig_mask, ScreenPaintData )
         // preparation step
         effects->prePaintWindow( effectWindow( w ), data, time_diff );
 #ifndef NDEBUG
-        foreach( WindowQuad q, data.quads )
+        foreach( const WindowQuad &q, data.quads )
             if( q.isTransformed())
                 kFatal( 1212 ) << "Pre-paint calls are not allowed to transform quads!" ;
 #endif
         if( !w->isPaintingEnabled())
             continue;
         phase2.append( Phase2Data( w, infiniteRegion(), data.clip, data.mask, data.quads ));
+        // transformations require window pixmap
+        w->suspendUnredirect( data.mask
+            & ( PAINT_WINDOW_TRANSLUCENT | PAINT_SCREEN_TRANSFORMED | PAINT_WINDOW_TRANSFORMED )); 
         }
 
-    foreach( Phase2Data d, phase2 )
+    foreach( const Phase2Data &d, phase2 )
         paintWindow( d.window, d.mask, d.region, d.quads );
     }
 
@@ -210,6 +218,7 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
     // TODO PAINT_WINDOW_* flags don't belong here, that's why it's in the assert,
     // perhaps the two enums should be separated
     assert(( orig_mask & ( PAINT_WINDOW_TRANSFORMED | PAINT_SCREEN_TRANSFORMED
+        | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS
         | PAINT_WINDOW_TRANSLUCENT | PAINT_WINDOW_OPAQUE )) == 0 );
     QHash< Window*, Phase2Data > phase2data;
     // Draw each opaque window top to bottom, subtracting the bounding rect of
@@ -223,21 +232,29 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
         data.mask = orig_mask | ( w->isOpaque() ? PAINT_WINDOW_OPAQUE : PAINT_WINDOW_TRANSLUCENT );
         w->resetPaintingEnabled();
         data.paint = region;
-        data.clip = w->isOpaque() ? w->shape().translated( w->x(), w->y()) : QRegion();
+        // Clip out the decoration for opaque windows; the decoration is drawn in the second pass
+        data.clip = w->isOpaque() ? w->clientShape().translated( w->x(), w->y() ) : QRegion();
         data.quads = w->buildQuads();
         // preparation step
         effects->prePaintWindow( effectWindow( w ), data, time_diff );
 #ifndef NDEBUG
-        foreach( WindowQuad q, data.quads )
+        foreach( const WindowQuad &q, data.quads )
             if( q.isTransformed())
                 kFatal( 1212 ) << "Pre-paint calls are not allowed to transform quads!" ;
+        if( data.mask & PAINT_WINDOW_TRANSFORMED )
+            kFatal( 1212 ) << "PAINT_WINDOW_TRANSFORMED without PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS!";
 #endif
         if( !w->isPaintingEnabled())
+            {
+            w->suspendUnredirect( true );
             continue;
+            }
         if( data.paint != region ) // prepaint added area to draw
             painted_region |= data.paint; // make sure it makes it to the screen
         // Schedule the window for painting
         phase2data[w] = Phase2Data( w, data.paint, data.clip, data.mask, data.quads );
+        // no transformations, but translucency requires window pixmap
+        w->suspendUnredirect( data.mask & PAINT_WINDOW_TRANSLUCENT );
         }
     // Do the actual painting
     // First opaque windows, top to bottom
@@ -264,6 +281,9 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
             {
             // Paint the opaque window
             paintWindow( d.window, d.mask, d.region, d.quads );
+            // Clip out the client area, so we only draw the decoration in the next pass
+            phase2data[w].region = d.region - d.clip;
+            phase2data[w].mask |= PAINT_DECORATION_ONLY;
             }
         }
     // Fill any areas of the root window not covered by windows
@@ -276,8 +296,7 @@ void Scene::paintSimpleScreen( int orig_mask, QRegion region )
         if( !phase2data.contains( w ))
             continue;
         Phase2Data d = phase2data[w];
-        if( d.mask & PAINT_WINDOW_TRANSLUCENT )
-            paintWindow( d.window, d.mask, d.region, d.quads );
+        paintWindow( d.window, d.mask, d.region, d.quads );
         }
     }
 
@@ -305,6 +324,31 @@ void Scene::finalDrawWindow( EffectWindowImpl* w, int mask, QRegion region, Wind
     w->sceneWindow()->performPaint( mask, region, data );
     }
 
+QList< QPoint > Scene::selfCheckPoints() const
+    {
+    QList< QPoint > ret;
+    // Use Kephal directly, we're interested in "real" screens, not depending on our config.
+    // TODO: Does Kephal allow fake screens as well? We cannot use QDesktopWidget as it will cause a crash if
+    //       the number of screens is different to what Kephal returns.
+    for( int screen = 0;
+         screen < Kephal::ScreenUtils::numScreens();
+         ++screen )
+        { // test top-left and bottom-right of every screen
+        ret.append( Kephal::ScreenUtils::screenGeometry( screen ).topLeft());
+        ret.append( Kephal::ScreenUtils::screenGeometry( screen ).bottomRight() + QPoint( -3 + 1, -2 + 1 )
+            + QPoint( -1, 0 )); // intentionally moved one up, since the source windows will be one down
+        }
+    return ret;
+    }
+
+QRegion Scene::selfCheckRegion() const
+    {
+    QRegion reg;
+    foreach( const QPoint& p, selfCheckPoints())
+        reg |= QRect( p, QSize( selfCheckWidth(), selfCheckHeight()));
+    return reg;
+    }
+
 //****************************************
 // Scene::Window
 //****************************************
@@ -353,6 +397,8 @@ QRegion Scene::Window::shape() const
                     shape_region += QRegion( rects[ i ].x, rects[ i ].y,
                         rects[ i ].width, rects[ i ].height );
                 XFree(rects);
+                // make sure the shape is sane (X is async, maybe even XShape is broken)
+                shape_region &= QRegion( 0, 0, width(), height());
                 }
             else
                 shape_region = QRegion();
@@ -364,6 +410,16 @@ QRegion Scene::Window::shape() const
     return shape_region;
     }
 
+QRegion Scene::Window::clientShape() const
+    {
+    Client *c = dynamic_cast< Client* >( toplevel );
+    if ( c && c->isShade() )
+        return QRegion();
+
+    const QRegion r = shape() & QRect( toplevel->clientPos(), toplevel->clientSize() );
+    return r.isEmpty() ? QRegion() : r;
+    }
+
 bool Scene::Window::isVisible() const
     {
     if( dynamic_cast< Deleted* >( toplevel ) != NULL )
@@ -375,7 +431,7 @@ bool Scene::Window::isVisible() const
     return true; // Unmanaged is always visible
     // TODO there may be transformations, so ignore this for now
     return !toplevel->geometry()
-        .intersect( QRect( 0, 0, displayWidth(), displayHeight()))
+        .intersected( QRect( 0, 0, displayWidth(), displayHeight()))
         .isEmpty();
     }
 
@@ -415,20 +471,34 @@ void Scene::Window::disablePainting( int reason )
     disable_painting |= reason;
     }
 
-WindowQuadList Scene::Window::buildQuads() const
+WindowQuadList Scene::Window::buildQuads( bool force ) const
     {
-    if( cached_quad_list != NULL )
+    if( cached_quad_list != NULL && !force )
         return *cached_quad_list;
     WindowQuadList ret;
     if( toplevel->clientPos() == QPoint( 0, 0 ) && toplevel->clientSize() == toplevel->size())
         ret = makeQuads( WindowQuadContents, shape()); // has no decoration
     else
         {
-        QRegion contents = shape() & QRect( toplevel->clientPos(), toplevel->clientSize());
-        QRegion decoration = shape() - contents;
+        Client *client = dynamic_cast<Client*>( toplevel );
+        QRegion contents = clientShape();
+        QRegion decoration = (client && Workspace::self()->decorationHasAlpha() ?
+                              QRegion(client->decorationRect()) : shape()) - contents;
         ret = makeQuads( WindowQuadContents, contents );
-        ret += makeQuads( WindowQuadDecoration, decoration );
+        if( (client && !client->isShade()) || !client )
+            ret += makeQuads( WindowQuadDecoration, decoration );
+        else
+            {
+            // this is a shaded client, we have to create four decoartion quads
+            QRect left, top, right, bottom;
+            client->layoutDecorationRects( left, top, right, bottom, Client::WindowRelative );
+            ret += makeQuads( WindowQuadDecoration, top );
+            ret += makeQuads( WindowQuadDecoration, bottom );
+            ret += makeQuads( WindowQuadDecoration, left );
+            ret += makeQuads( WindowQuadDecoration, right );
+            }
         }
+    effects->buildQuads( static_cast<Client*>( toplevel )->effectWindow(), ret );
     cached_quad_list = new WindowQuadList( ret );
     return ret;
     }
@@ -436,7 +506,7 @@ WindowQuadList Scene::Window::buildQuads() const
 WindowQuadList Scene::Window::makeQuads( WindowQuadType type, const QRegion& reg ) const
     {
     WindowQuadList ret;
-    foreach( QRect r, reg.rects())
+    foreach( const QRect &r, reg.rects())
         {
         WindowQuad quad( type );
         // TODO asi mam spatne pravy dolni roh - bud tady, nebo v jinych castech
