@@ -109,16 +109,13 @@ QMenu* Workspace::clientPopup()
         action->setData( Options::SetupWindowShortcutOp );
 
         action = advanced_popup->addAction( i18n("&Special Window Settings...") );
-        action->setIcon( KIcon( "wizard" ) );
+        action->setIcon( KIcon( "preferences-system-windows-actions" ) );
         action->setData( Options::WindowRulesOp );
 
         action = advanced_popup->addAction( i18n("&Special Application Settings...") );
-        action->setIcon( KIcon( "wizard" ) );
+        action->setIcon( KIcon( "preferences-system-windows-actions" ) );
         action->setData( Options::ApplicationRulesOp );
 
-        action = popup->addMenu( advanced_popup );
-        action->setText( i18n("Ad&vanced") );
-
         trans_popup = 0;
         if (compositing()){
             trans_popup = new QMenu( popup );
@@ -170,6 +167,9 @@ QMenu* Workspace::clientPopup()
         mShadeOpAction->setCheckable( true );
         mShadeOpAction->setData( Options::ShadeOp );
 
+        action = popup->addMenu( advanced_popup );
+        action->setText( i18n("Ad&vanced") );
+
         popup->addSeparator();
 
         if (!KGlobal::config()->isImmutable() &&
@@ -191,6 +191,13 @@ QMenu* Workspace::clientPopup()
     return popup;
     }
 
+void Workspace::discardPopup()
+    {
+    delete popup;
+    popup = NULL; 
+    desk_popup = NULL;
+    }
+
 void Workspace::setPopupClientOpacity( QAction* action )
     {
     if( active_popup_client == NULL )
@@ -220,7 +227,7 @@ void Workspace::clientPopupAboutToShow()
         }
 
     mResizeOpAction->setEnabled( active_popup_client->isResizable() );
-    mMoveOpAction->setEnabled( active_popup_client->isMovable() );
+    mMoveOpAction->setEnabled( active_popup_client->isMovableAcrossScreens() );
     mMaximizeOpAction->setEnabled( active_popup_client->isMaximizable() );
     mMaximizeOpAction->setChecked( active_popup_client->maximizeMode() == Client::MaximizeFull );
     mShadeOpAction->setEnabled( active_popup_client->isShadeable() );
@@ -259,7 +266,8 @@ void Workspace::initDesktopPopup()
              this, SLOT( desktopPopupAboutToShow() ) );
 
     QAction *action = desk_popup->menuAction();
-    popup->insertAction(mMoveOpAction, action);
+    // set it as the first item
+    popup->insertAction( trans_popup ? trans_popup->menuAction() : mMoveOpAction, action);
     action->setText( i18n("To &Desktop") );
     }
 
@@ -338,31 +346,46 @@ void Workspace::readShortcuts()
 
     KAction *kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktops") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktops = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkThroughDesktopsKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktops (Reverse)") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktopsReverse = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughDesktopsKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktop List") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktopList = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkThroughDesktopListKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktop List (Reverse)") );
     if ( kaction!=0 )
+        {
         cutWalkThroughDesktopListReverse = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughDesktopListKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Windows") );
     if ( kaction!=0 )
+        {
         cutWalkThroughWindows = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkThroughWindowsKeyChanged(QKeySequence)));
+        }
 
     kaction = qobject_cast<KAction*>( keys->action("Walk Through Windows (Reverse)") );
     if ( kaction!=0 )
+        {
         cutWalkThroughWindowsReverse = kaction->globalShortcut();
-
-    delete popup;
-    popup = NULL; // so that it's recreated next time
-    desk_popup = NULL;
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughWindowsKeyChanged(QKeySequence)));
+        }
+    discardPopup(); // so that it's recreated next time
     }
 
 
@@ -399,23 +422,38 @@ void Workspace::setupWindowShortcutDone( bool ok )
     if( ok )
         client_keys_client->setShortcut( KShortcut( client_keys_dialog->shortcut()).toString());
     closeActivePopup();
-    delete client_keys_dialog;
+    client_keys_dialog->deleteLater();
     client_keys_dialog = NULL;
     client_keys_client = NULL;
     }
 
 void Workspace::clientShortcutUpdated( Client* c )
     {
-    QString key = QString::number( c->window());
+    QString key = QString( "_k_session:%1" ).arg(c->window());
     QAction* action = client_keys->action( key.toLatin1().constData() );
     if( !c->shortcut().isEmpty())
         {
-        action->setShortcuts(c->shortcut());
-        connect(action, SIGNAL(triggered(bool)), c, SLOT(shortcutActivated()));
-        action->setEnabled( true );
+        if( action == NULL ) // new shortcut
+            {
+            action = client_keys->addAction(QString( key ));
+            action->setText( i18n("Activate Window (%1)", c->caption()) );
+            connect( action, SIGNAL(triggered(bool)), c, SLOT(shortcutActivated()) );
+            }
+
+        KAction *kaction = qobject_cast<KAction*>( action );
+        // no autoloading, since it's configured explicitly here and is not meant to be reused
+        // (the key is the window id anyway, which is kind of random)
+        kaction->setGlobalShortcut(
+                c->shortcut(), KAction::ActiveShortcut, KAction::NoAutoloading );
+        kaction->setEnabled( true );
         }
     else
         {
+        KAction *kaction = qobject_cast<KAction*>( action );
+        if( kaction )
+            {
+            kaction->forgetGlobalShortcut();
+            }
         delete action;
         }
     }
@@ -447,7 +485,7 @@ void Workspace::clientPopupActivated( QAction *action )
     }
 
 
-void Workspace::performWindowOperation( Client* c, Options::WindowOperation op ) 
+void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
     {
     if ( !c )
         return;
@@ -456,7 +494,7 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
         QCursor::setPos( c->geometry().center() );
     if (op == Options::ResizeOp || op == Options::UnrestrictedResizeOp )
         QCursor::setPos( c->geometry().bottomRight());
-    switch ( op ) 
+    switch ( op )
         {
         case Options::MoveOp:
             c->performMouseCommand( Options::MouseMove, cursorPos() );
@@ -499,7 +537,7 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
             c->setFullScreen( !c->isFullScreen(), true );
             break;
         case Options::NoBorderOp:
-            c->setUserNoBorder( !c->isUserNoBorder());
+            c->setNoBorder( !c->noBorder());
             break;
         case Options::KeepAboveOp:
             {
@@ -545,25 +583,29 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
 bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &globalPos, bool handled )
     {
     bool replay = false;
-    switch (command) 
+    switch (command)
         {
         case Options::MouseRaise:
             workspace()->raiseClient( this );
             break;
         case Options::MouseLower:
             workspace()->lowerClient( this );
+            // As this most likely makes the window no longer visible change the
+            // keyboard focus to the next available window.
+            //workspace()->activateNextClient( this ); // Doesn't work when we lower a child window
+            workspace()->activateClient( workspace()->topClientOnDesktop( workspace()->currentDesktop(), -1 ));
             break;
         case Options::MouseShade :
             toggleShade();
-            cancelShadeHover();
+            cancelShadeHoverTimer();
             break;
         case Options::MouseSetShade:
             setShade( ShadeNormal );
-            cancelShadeHover();
+            cancelShadeHoverTimer();
             break;
         case Options::MouseUnsetShade:
             setShade( ShadeNone );
-            cancelShadeHover();
+            cancelShadeHoverTimer();
             break;
         case Options::MouseOperationsMenu:
             if ( isActive() && options->clickRaise )
@@ -603,13 +645,13 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             workspace()->raiseClient( this );
             workspace()->requestFocus( this );
             workspace()->setActiveScreenMouse( globalPos );
-            if( options->moveMode == Options::Transparent && isMovable())
+            if( options->moveMode == Options::Transparent && isMovableAcrossScreens())
                 move_faked_activity = workspace()->fakeRequestedActivity( this );
         // fallthrough
         case Options::MouseMove:
         case Options::MouseUnrestrictedMove:
             {
-            if (!isMovable())
+            if (!isMovableAcrossScreens())
                 break;
             if( moveResizeMode )
                 finishMoveResize( false );
@@ -685,10 +727,12 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             workspace()->windowToNextDesktop( this );
             break;
         case Options::MouseOpacityMore:
-            setOpacity( qMin( opacity() + 0.1, 1.0 ));
+            if( !isDesktop() ) // No point in changing the opacity of the desktop
+                setOpacity( qMin( opacity() + 0.1, 1.0 ));
             break;
         case Options::MouseOpacityLess:
-            setOpacity( qMax( opacity() - 0.1, 0.0 ));
+            if( !isDesktop() ) // No point in changing the opacity of the desktop
+                setOpacity( qMax( opacity() - 0.1, 0.1 ));
             break;
         case Options::MouseNothing:
             replay = true;
@@ -721,10 +765,26 @@ void Workspace::unloadEffect( const QString& name )
         static_cast<EffectsHandlerImpl*>(effects)->unloadEffect( name );
     }
 
-void Workspace::reloadEffect( const QString& name )
+void Workspace::reconfigureEffect( const QString& name )
     {
     if( effects )
-        static_cast<EffectsHandlerImpl*>(effects)->reloadEffect( name );
+        static_cast<EffectsHandlerImpl*>(effects)->reconfigureEffect( name );
+    }
+
+QStringList Workspace::loadedEffects() const
+    {
+    QStringList listModulesLoaded;
+    if ( effects )
+        listModulesLoaded = static_cast<EffectsHandlerImpl*>(effects)->loadedEffects();
+    return listModulesLoaded;
+    }
+
+QStringList Workspace::listOfEffects() const
+    {
+    QStringList listModules;
+    if ( effects )
+        listModules = static_cast<EffectsHandlerImpl*>(effects)->listOfEffects();
+    return listModules;
     }
 
 void Workspace::slotActivateAttentionWindow()
@@ -736,13 +796,13 @@ void Workspace::slotActivateAttentionWindow()
 void Workspace::slotSwitchDesktopNext()
     {
     int d = currentDesktop() + 1;
-     if ( d > numberOfDesktops() ) 
+     if ( d > numberOfDesktops() )
         {
-        if ( options->rollOverDesktops ) 
+        if ( options->rollOverDesktops )
             {
             d = 1;
             }
-        else 
+        else
             {
             return;
             }
@@ -753,7 +813,7 @@ void Workspace::slotSwitchDesktopNext()
 void Workspace::slotSwitchDesktopPrevious()
     {
     int d = currentDesktop() - 1;
-    if ( d <= 0 ) 
+    if ( d <= 0 )
         {
         if ( options->rollOverDesktops )
           d = numberOfDesktops();
@@ -781,7 +841,7 @@ void Workspace::slotSwitchDesktopLeft()
 
 void Workspace::slotSwitchDesktopUp()
     {
-    int desktop = desktopUp( currentDesktop(), options->rollOverDesktops);
+    int desktop = desktopAbove( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -789,7 +849,7 @@ void Workspace::slotSwitchDesktopUp()
 
 void Workspace::slotSwitchDesktopDown()
     {
-    int desktop = desktopDown( currentDesktop(), options->rollOverDesktops);
+    int desktop = desktopBelow( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -911,7 +971,13 @@ void Workspace::slotWindowLower()
     {
     Client* c = active_popup_client ? active_popup_client : active_client;
     if ( c )
+        {
         lowerClient( c );
+        // As this most likely makes the window no longer visible change the
+        // keyboard focus to the next available window.
+        //activateNextClient( c ); // Doesn't work when we lower a child window
+        activateClient( topClientOnDesktop( currentDesktop(), -1 ));
+        }
     }
 
 /*!
@@ -972,7 +1038,7 @@ void Workspace::slotWindowToNextDesktop()
     {
     windowToNextDesktop( active_popup_client ? active_popup_client : active_client );
     }
-    
+
 void Workspace::windowToNextDesktop( Client* c )
     {
     int d = currentDesktop() + 1;
@@ -994,7 +1060,7 @@ void Workspace::slotWindowToPreviousDesktop()
     {
     windowToPreviousDesktop( active_popup_client ? active_popup_client : active_client );
     }
-    
+
 void Workspace::windowToPreviousDesktop( Client* c )
     {
     int d = currentDesktop() - 1;
@@ -1041,7 +1107,7 @@ void Workspace::slotWindowToDesktopLeft()
 
 void Workspace::slotWindowToDesktopUp()
     {
-    int d = desktopUp( currentDesktop(), options->rollOverDesktops);
+    int d = desktopAbove( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
@@ -1056,7 +1122,7 @@ void Workspace::slotWindowToDesktopUp()
 
 void Workspace::slotWindowToDesktopDown()
     {
-    int d = desktopDown( currentDesktop(), options->rollOverDesktops);
+    int d = desktopBelow( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
@@ -1089,7 +1155,7 @@ void Workspace::slotSendToDesktop( QAction *action )
       int desk = action->data().toInt();
     if ( !active_popup_client )
         return;
-    if ( desk == 0 ) 
+    if ( desk == 0 )
         { // the 'on_all_desktops' menu entry
         active_popup_client->setOnAllDesktops( !active_popup_client->isOnAllDesktops());
         return;
@@ -1192,8 +1258,8 @@ void Client::setShortcut( const QString& _cut )
         }
     QList< KShortcut > keys;
     QStringList groups = cut.split( ' ');
-    for( QStringList::ConstIterator it = groups.begin();
-         it != groups.end();
+    for( QStringList::ConstIterator it = groups.constBegin();
+         it != groups.constEnd();
          ++it )
         {
         QRegExp reg( "(.*\\+)\\((.*)\\)" );
@@ -1211,15 +1277,15 @@ void Client::setShortcut( const QString& _cut )
                 }
             }
         }
-    for( QList< KShortcut >::ConstIterator it = keys.begin();
-         it != keys.end();
+    for( QList< KShortcut >::ConstIterator it = keys.constBegin();
+         it != keys.constEnd();
          ++it )
         {
         if( _shortcut == *it ) // current one is in the list
             return;
         }
-    for( QList< KShortcut >::ConstIterator it = keys.begin();
-         it != keys.end();
+    for( QList< KShortcut >::ConstIterator it = keys.constBegin();
+         it != keys.constEnd();
          ++it )
         {
         if( workspace()->shortcutAvailable( *it, this ))
@@ -1237,18 +1303,30 @@ void Client::setShortcutInternal( const KShortcut& cut )
         return;
     _shortcut = cut;
     updateCaption();
+#if 0
+    workspace()->clientShortcutUpdated( this );
+#else
+    // Workaround for kwin<->kglobalaccel deadlock, when KWin has X grab and the kded
+    // kglobalaccel module tries to create the key grab. KWin should preferably grab
+    // they keys itself anyway :(.
+    QTimer::singleShot( 0, this, SLOT( delayedSetShortcut()));
+#endif
+    }
+
+void Client::delayedSetShortcut()
+    {
     workspace()->clientShortcutUpdated( this );
     }
 
 bool Workspace::shortcutAvailable( const KShortcut& cut, Client* ignore ) const
     {
     // TODO check global shortcuts etc.
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it )
         {
         if( (*it) != ignore && (*it)->shortcut() == cut )
-            return false;    
+            return false;
         }
     return true;
     }
