@@ -21,6 +21,8 @@
 
 */
 
+#include "../config-ksysguard.h"
+
 #include <QTimer>
 #include <QList>
 #include <QShowEvent>
@@ -31,7 +33,7 @@
 #include <QMenu>
 #include <QSet>
 #include <QComboBox>
-#include <QItemDelegate>
+#include <QStyledItemDelegate>
 #include <QPainter>
 #include <QProcess>
 #include <QLineEdit>
@@ -54,52 +56,46 @@
 #include "ReniceDlg.h"
 #include "ui_ProcessWidgetUI.h"
 
+#ifdef WITH_MONITOR_PROCESS_IO
+#include "DisplayProcessDlg.h"
+#endif
+
+#include <sys/types.h>
+#include <unistd.h>
+
 //Trolltech have a testing class for classes that inherit QAbstractItemModel.  If you want to run with this run-time testing enabled, put the modeltest.* files in this directory and uncomment the next line
 //#define DO_MODELCHECK
 #ifdef DO_MODELCHECK
 #include "modeltest.h"
 #endif
 
-class ProgressBarItemDelegate : public QItemDelegate 
+class ProgressBarItemDelegate : public QStyledItemDelegate
 {
   public:
-	ProgressBarItemDelegate(QObject *parent) : QItemDelegate(parent), startProgressColor(0x00, 0x71, 0xBC, 100), endProgressColor(0x83, 0xDD, 0xF5, 100), totalMemory(-1), numCpuCores(-1) {}
-  protected:
-	virtual void drawDisplay(QPainter *painter, const QStyleOptionViewItem &option,
-		                                 const QRect &rect, const QString &text) const
-	{
-		
-		if(percentage > 0 && percentage * rect.width() > 100 ) { //make sure the line will have a width of more than 1 pixel
-			QPen old = painter->pen();
-			painter->setPen(Qt::NoPen);
-			QLinearGradient  linearGrad( QPointF(rect.x(),rect.y()), QPointF(rect.x() + rect.width(), rect.y()));
-			linearGrad.setColorAt(0, startProgressColor);
-			linearGrad.setColorAt(1, endProgressColor);
-			painter->fillRect( rect.x(), rect.y(), rect.width() * percentage /100 , rect.height(), QBrush(linearGrad));
-			painter->setPen( old );
-		}
-		
-		QItemDelegate::drawDisplay( painter, option, rect, text);
-	}
-	void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+	ProgressBarItemDelegate(QObject *parent) : QStyledItemDelegate(parent), startProgressColor(0x00, 0x71, 0xBC, 100), endProgressColor(0x83, 0xDD, 0xF5, 100), totalMemory(-1), numCpuCores(-1) {}
+
+  	virtual void paint(QPainter *painter, const QStyleOptionViewItem &opt, const QModelIndex &index) const
 	{
+		QStyleOptionViewItemV4 option = opt;
+		initStyleOption(&option,index);
+
 		Q_ASSERT(index.model());
 		QModelIndex realIndex = (reinterpret_cast< const QAbstractProxyModel *> (index.model()))->mapToSource(index);
 		KSysGuard::Process *process = reinterpret_cast< KSysGuard::Process * > (realIndex.internalPointer());
 		Q_CHECK_PTR(process);
 		if(index.column() == ProcessModel::HeadingCPUUsage) {
-			if(numCpuCores == -1) 
+			if(numCpuCores == -1)
 				numCpuCores = index.data(ProcessModel::NumberOfProcessorsRole).toInt();
 			percentage = (process->userUsage + process->sysUsage) / numCpuCores;
 		} else if(index.column() == ProcessModel::HeadingMemory) {
 			long long memory = 0;
-			if(process->vmURSS != -1) 
+			if(process->vmURSS != -1)
 				memory = process->vmURSS;
-			else 
+			else
 				memory = process->vmRSS;
 			if(totalMemory == -1)
 				totalMemory = index.data(ProcessModel::TotalMemoryRole).toLongLong();
-			if(totalMemory > 0) 
+			if(totalMemory > 0)
 				percentage = (int)(memory*100/totalMemory);
 			else
 				percentage = 0;
@@ -113,55 +109,83 @@ class ProgressBarItemDelegate : public QItemDelegate
 					percentage = 0;
 			}
 		} else
-			percentage = 0;
-		QItemDelegate::paint(painter, option, index);
+			percentage = -1;
+
+		if (percentage >= 0)
+			drawPercentageDisplay(painter,option,index.data(Qt::DisplayRole).toString());
+		else 
+			QStyledItemDelegate::paint(painter, option, index);
 	}
+
+  private:
+	void drawPercentageDisplay(QPainter *painter, const QStyleOptionViewItem& option, const QString& text) const
+	{
+		QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem,&option,painter);
+
+		const QRect rect = option.rect;
+		if(percentage * rect.width() > 100 ) { //make sure the line will have a width of more than 1 pixel
+			if(percentage > 100) 
+				percentage = 100;  //Don't draw outside our bounds
+			QPen old = painter->pen();
+			painter->setPen(Qt::NoPen);
+			QLinearGradient  linearGrad( QPointF(rect.x(),rect.y()), QPointF(rect.x() + rect.width(), rect.y()));
+			linearGrad.setColorAt(0, startProgressColor);
+			linearGrad.setColorAt(1, endProgressColor);
+			painter->fillRect( rect.x(), rect.y(), rect.width() * percentage /100 , rect.height(), QBrush(linearGrad));
+			painter->setPen( old );
+		}
+		painter->drawText(rect,option.displayAlignment,text + ' ');
+	}
+
 	mutable int percentage;
 	QColor startProgressColor;
 	QColor endProgressColor;
 	mutable long long totalMemory;
 	mutable int numCpuCores;
-
 };
 
 struct KSysGuardProcessListPrivate {
-    
-	KSysGuardProcessListPrivate(KSysGuardProcessList* q, const QString &hostName) 
+
+	KSysGuardProcessListPrivate(KSysGuardProcessList* q, const QString &hostName)
             : mModel(q, hostName), mFilterModel(q), mUi(new Ui::ProcessWidget()), mProcessContextMenu(NULL), mUpdateTimer(NULL)
         {
 		renice = new QAction(i18np("Renice Process...", "Renice Processes...", 1), q);
 		selectParent = new QAction(i18n("Jump to Parent Process"), q);
 
-		selectTracer = new QAction(i18n("Jump to process debugging this one"), q);
-		window = new QAction(i18n("Show application window"), q);
-		resume = new QAction(i18n("Resume stopped process"), q);
+		selectTracer = new QAction(i18n("Jump to Process Debugging This One"), q);
+		window = new QAction(i18n("Show Application Window"), q);
+#ifdef WITH_MONITOR_PROCESS_IO
+		monitorio = new QAction(i18n("Monitor Input && Output"), q);
+#else
+		monitorio = 0;
+#endif
+		resume = new QAction(i18n("Resume Stopped Process"), q);
 		kill = new QAction(i18np("Kill Process", "Kill Processes", 1), q);
 		kill->setIcon(KIcon("process-stop"));
 
 		sigStop = new QAction(i18n("Suspend (STOP)"), q);
 		sigCont = new QAction(i18n("Continue (CONT)"), q);
 		sigHup = new QAction(i18n("Hangup (HUP)"), q);
-		sigInt = new QAction(i18n("Interupt (INT)"), q);
+		sigInt = new QAction(i18n("Interrupt (INT)"), q);
 		sigTerm = new QAction(i18n("Terminate (TERM)"), q);
 		sigKill = new QAction(i18n("Kill (KILL)"), q);
 		sigUsr1 = new QAction(i18n("User 1 (USR1)"), q);
 		sigUsr2 = new QAction(i18n("User 2 (USR2)"), q);
-		
 	}
 
         ~KSysGuardProcessListPrivate() { delete mUi; mUi = NULL; }
-	
+
 	/** The process model.  This contains all the data on all the processes running on the system */
 	ProcessModel mModel;
-	
+
 	/** The process filter.  The mModel is connected to this, and this filter model connects to the view.  This lets us
 	 *  sort the view and filter (by using the combo box or the search line)
 	 */
 	ProcessFilter mFilterModel;
-	
+
 	/** The graphical user interface for this process list widget, auto-generated by Qt Designer */
 	Ui::ProcessWidget *mUi;
-	
+
 	/** The context menu when you right click on a process */
 	QMenu *mProcessContextMenu;
 
@@ -176,6 +200,7 @@ struct KSysGuardProcessListPrivate {
 	QAction *selectParent;
 	QAction *selectTracer;
 	QAction *window;
+	QAction *monitorio;
 	QAction *resume;
 	QAction *sigStop;
 	QAction *sigCont;
@@ -208,7 +233,7 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 
 	d->mUi->treeView->header()->setClickable(true);
 	d->mUi->treeView->header()->setSortIndicatorShown(true);
-	d->mUi->treeView->header()->setCascadingSectionResizes(true);
+	d->mUi->treeView->header()->setCascadingSectionResizes(false);
 	connect(d->mUi->btnKillProcess, SIGNAL(clicked()), this, SLOT(killSelectedProcesses()));
 	connect(d->mUi->txtFilter, SIGNAL(textChanged(const QString &)), this, SLOT(filterTextChanged(const QString &)));
 	connect(d->mUi->cmbFilter, SIGNAL(currentIndexChanged(int)), this, SLOT(setStateInt(int)));
@@ -217,17 +242,27 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 	connect(&d->mFilterModel, SIGNAL(rowsInserted( const QModelIndex &, int, int)), this, SLOT(rowsInserted(const QModelIndex &, int, int)));
 	setMinimumSize(sizeHint());
 
+	d->mFilterModel.setFilterKeyColumn(-1);
+
+
 	/*  Hide the vm size column by default since it's not very useful */
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingVmSize);
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingNiceness);
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingTty);
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingCommand);
 	d->mUi->treeView->header()->hideSection(ProcessModel::HeadingPid);
-	d->mFilterModel.setFilterKeyColumn(0);
-
+	// NOTE!  After this is all setup, the settings for the header are restored
+	// from the user's last run.  (in restoreHeaderState)
+	// So making changes here only affects the default settings.  To 
+	// test changes temporarily, comment out the lines in restoreHeaderState.
+	// When you are happy with the changes and want to commit, increase the 
+	// value of PROCESSHEADERVERSION.  This will force the header state
+	// to be reset back to the defaults for all users.
 	d->mUi->treeView->header()->resizeSection(ProcessModel::HeadingCPUUsage, d->mUi->treeView->header()->sectionSizeHint(ProcessModel::HeadingCPUUsage));
 	d->mUi->treeView->header()->resizeSection(ProcessModel::HeadingMemory, d->mUi->treeView->header()->sectionSizeHint(ProcessModel::HeadingMemory));
 	d->mUi->treeView->header()->resizeSection(ProcessModel::HeadingSharedMemory, d->mUi->treeView->header()->sectionSizeHint(ProcessModel::HeadingSharedMemory));
+	d->mUi->treeView->header()->setResizeMode(0, QHeaderView::Interactive);
+	d->mUi->treeView->header()->setStretchLastSection(true);
 
 	//Process names can have mixed case. Make the filter case insensitive.
 	d->mFilterModel.setFilterCaseSensitivity(Qt::CaseInsensitive);
@@ -236,20 +271,22 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 	d->mUi->txtFilter->installEventFilter(this);
 	d->mUi->treeView->installEventFilter(this);
 
-	//Logical column 0 will always be the tree bit with the process name.  We expand this automatically in code,
-	//so don't let the user change it
-	d->mUi->treeView->header()->setResizeMode(0, QHeaderView::ResizeToContents);
-	d->mUi->treeView->header()->setStretchLastSection(true);
+
+	d->mUi->treeView->setDragEnabled(true);
+	d->mUi->treeView->setDragDropMode(QAbstractItemView::DragOnly);
+
 
 	//Sort by username by default
 	d->mUi->treeView->sortByColumn(ProcessModel::HeadingUser, Qt::AscendingOrder);
 	d->mFilterModel.sort(ProcessModel::HeadingUser, Qt::AscendingOrder);
 
-	// Add all the actions to the main widget, and get all the actions to call actionTriggered when clicked	
+	// Add all the actions to the main widget, and get all the actions to call actionTriggered when clicked
 	QSignalMapper *signalMapper = new QSignalMapper(this);
 	QList<QAction *> actions;
-	actions << d->renice << d->kill << d->selectParent << d->selectTracer << d->window << d->resume;
-	actions << d->sigStop << d->sigCont << d->sigHup << d->sigInt << d->sigTerm << d->sigKill << d->sigUsr1 << d->sigUsr2;
+	actions << d->renice << d->kill << d->selectParent << d->selectTracer << d->window;
+	if (d->monitorio)
+	    actions << d->monitorio;
+	actions << d->resume << d->sigStop << d->sigCont << d->sigHup << d->sigInt << d->sigTerm << d->sigKill << d->sigUsr1 << d->sigUsr2;
 	foreach(QAction *action, actions) {
 		addAction(action);
 		connect(action, SIGNAL(triggered(bool)), signalMapper, SLOT(map()));
@@ -259,21 +296,20 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 
 	retranslateUi();
 
-	// Dynamic sort filter seems to require repainting the whole screen, slowing everything down drastically.
-	// When this bug is fixed we can re-enable this.
-	//d->mFilterModel.setDynamicSortFilter(true);
+	d->mFilterModel.setDynamicSortFilter(true);
 
 	d->mUpdateTimer = new QTimer(this);
 	d->mUpdateTimer->setSingleShot(true);
 	connect(d->mUpdateTimer, SIGNAL(timeout()), this, SLOT(updateList()));
-	d->mUpdateTimer->start(0);
+	d->mUpdateTimer->start(d->mUpdateIntervalMSecs);
 
-	d->mUi->btnKillProcess->setIcon(KIcon("process-stop"));
+	d->mUi->btnKillProcess->setIcon(KIcon("edit-bomb"));
 
 	//If the view resorts continually, then it can be hard to keep track of processes.  By doing it only every few seconds it reduces the 'jumping around'
-	QTimer *mTimer = new QTimer(this);
-	connect(mTimer, SIGNAL(timeout()), &d->mFilterModel, SLOT(invalidate()));
-	mTimer->start(4000); 
+//	QTimer *mTimer = new QTimer(this);
+//	mTimer->start(4000);
+// QT BUG?  We have to disable the sorting for now because there seems to be a bug in Qt introduced in Qt 4.4beta which makes the view scroll back to the top
+        d->mModel.update(d->mUpdateIntervalMSecs);
 }
 
 KSysGuardProcessList::~KSysGuardProcessList()
@@ -289,7 +325,7 @@ QLineEdit *KSysGuardProcessList::filterLineEdit() const {
 	return d->mUi->txtFilter;
 }
 
-ProcessFilter::State KSysGuardProcessList::state() const 
+ProcessFilter::State KSysGuardProcessList::state() const
 {
 	return d->mFilterModel.filter();
 }
@@ -329,7 +365,7 @@ void KSysGuardProcessList::showProcessContextMenu(const QPoint &point) {
 
 	QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
 	int numProcesses = selectedIndexes.size();
-        
+
 	if(numProcesses == 0) return;  //No processes selected, so no context menu
 
 	QModelIndex realIndex = d->mFilterModel.mapToSource(selectedIndexes.at(0));
@@ -353,9 +389,14 @@ void KSysGuardProcessList::showProcessContextMenu(const QPoint &point) {
 	}
 
 	if(numProcesses == 1 && process->parent_pid > 1) {
-		//As a design decision, I do not show the 'Jump to parent process' option when the 
+		//As a design decision, I do not show the 'Jump to parent process' option when the
 		//parent is just 'init'.
-		d->mProcessContextMenu->addAction(d->selectParent);
+
+	        KSysGuard::Process *parent_process = d->mModel.getProcess(process->parent_pid);
+		if(parent_process) { //it should not be possible for this process to not exist, but check just incase
+			d->selectParent->setText(i18n("Jump to Parent Process (%1)", parent_process->name));
+			d->mProcessContextMenu->addAction(d->selectParent);
+		}
 	}
 
 	if(numProcesses == 1 && process->tracerpid > 0) {
@@ -366,6 +407,9 @@ void KSysGuardProcessList::showProcessContextMenu(const QPoint &point) {
 	if (numProcesses == 1 && !d->mModel.data(realIndex, ProcessModel::WindowIdRole).isNull()) {
 		d->mProcessContextMenu->addAction(d->window);
 	}
+	if (d->monitorio && numProcesses == 1 && d->mModel.isLocalhost() && (process->uid==0 || process->uid == getuid()) && process->pid != getpid() && process->pid != getppid()) { //Don't attach to ourselves - crashes
+		d->mProcessContextMenu->addAction(d->monitorio);
+	}
 
 	if(numProcesses == 1 && process->status == KSysGuard::Process::Stopped) {
 		//If the process is being debugged, offer to select it
@@ -380,6 +424,8 @@ void KSysGuardProcessList::showProcessContextMenu(const QPoint &point) {
 	d->mProcessContextMenu->popup(d->mUi->treeView->viewport()->mapToGlobal(point));
 }
 void KSysGuardProcessList::actionTriggered(QObject *object) {
+	//Reset the text back to normal
+	d->selectParent->setText(i18n("Jump to Parent Process"));
 	QAction *result = dynamic_cast<QAction *>(object);
 	if(result == 0) {
 		//Escape was pressed. Do nothing.
@@ -389,7 +435,7 @@ void KSysGuardProcessList::actionTriggered(QObject *object) {
 		killSelectedProcesses();
 	} else if(result == d->selectParent) {
 		QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
-		int numProcesses = selectedIndexes.size();        
+		int numProcesses = selectedIndexes.size();
 		if(numProcesses == 0) return;  //No processes selected
 		QModelIndex realIndex = d->mFilterModel.mapToSource(selectedIndexes.at(0));
 		KSysGuard::Process *process = reinterpret_cast<KSysGuard::Process *> (realIndex.internalPointer());
@@ -397,7 +443,7 @@ void KSysGuardProcessList::actionTriggered(QObject *object) {
 			selectAndJumpToProcess(process->parent_pid);
 	} else if(result == d->selectTracer) {
 		QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
-		int numProcesses = selectedIndexes.size();        
+		int numProcesses = selectedIndexes.size();
 		if(numProcesses == 0) return;  //No processes selected
 		QModelIndex realIndex = d->mFilterModel.mapToSource(selectedIndexes.at(0));
 		KSysGuard::Process *process = reinterpret_cast<KSysGuard::Process *> (realIndex.internalPointer());
@@ -405,9 +451,9 @@ void KSysGuardProcessList::actionTriggered(QObject *object) {
 			selectAndJumpToProcess(process->tracerpid);
 	} else if(result == d->window) {
 		QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
-		int numProcesses = selectedIndexes.size();        
+		int numProcesses = selectedIndexes.size();
 		if(numProcesses == 0) return;  //No processes selected
-		foreach( QModelIndex index, selectedIndexes) {
+		foreach( const QModelIndex &index, selectedIndexes) {
 			QModelIndex realIndex = d->mFilterModel.mapToSource(index);
 			QVariant widVar= d->mModel.data(realIndex, ProcessModel::WindowIdRole);
 			if( !widVar.isNull() ) {
@@ -415,12 +461,23 @@ void KSysGuardProcessList::actionTriggered(QObject *object) {
 				KWindowSystem::activateWindow(wid);
 			}
 		}
+#ifdef WITH_MONITOR_PROCESS_IO
+	} else if(result == d->monitorio) {
+		QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
+		if(selectedIndexes.isEmpty()) return;  //No processes selected
+		QModelIndex realIndex = d->mFilterModel.mapToSource(selectedIndexes.at(0));
+		KSysGuard::Process *process = reinterpret_cast<KSysGuard::Process *> (realIndex.internalPointer());
+		if(process) {
+			DisplayProcessDlg *dialog = new DisplayProcessDlg( this,process );
+			dialog->show();
+		}
+#endif
 	} else {
 		QList< long long > pidlist;
 		QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
-		int numProcesses = selectedIndexes.size();        
+		int numProcesses = selectedIndexes.size();
 		if(numProcesses == 0) return;  //No processes selected
-		foreach( QModelIndex index, selectedIndexes) {
+		foreach( const QModelIndex &index, selectedIndexes) {
 			QModelIndex realIndex = d->mFilterModel.mapToSource(index);
 			KSysGuard::Process *process = reinterpret_cast<KSysGuard::Process *> (realIndex.internalPointer());
 			if(process)
@@ -449,36 +506,43 @@ void KSysGuardProcessList::actionTriggered(QObject *object) {
 void KSysGuardProcessList::selectAndJumpToProcess(int pid) {
 	KSysGuard::Process *process = d->mModel.getProcess(pid);
 	if(!process) return;
-	QModelIndex filterIndex = d->mFilterModel.mapFromSource( d->mModel.getQModelIndex(process, 0));
+	QModelIndex sourceIndex = d->mModel.getQModelIndex(process, 0);
+	QModelIndex filterIndex = d->mFilterModel.mapFromSource( sourceIndex );
+	if(!filterIndex.isValid() && !d->mUi->txtFilter->text().isEmpty()) {
+		//The filter is preventing us from finding the parent.  Clear the filter
+		//(It could also be the combo box - should we deal with that case as well?)
+		d->mUi->txtFilter->clear();
+		filterIndex = d->mFilterModel.mapFromSource( sourceIndex );
+	}
 	d->mUi->treeView->clearSelection();
 	d->mUi->treeView->setCurrentIndex(filterIndex);
 	d->mUi->treeView->scrollTo( filterIndex, QAbstractItemView::PositionAtCenter);
-	
+
 }
 
 void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
 	QMenu *menu = new QMenu();
-	
+
 	QAction *action;
 	int index = d->mUi->treeView->header()->logicalIndexAt(point);
 	if(index >= 0) {
 		//selected a column.  Give the option to hide it
 		action = new QAction(menu);
 		action->setData(-index-1); //We set data to be negative (and minus 1) to hide a column, and positive to show a column
-		action->setText(i18n("Hide column '%1'", d->mFilterModel.headerData(index, Qt::Horizontal, Qt::DisplayRole).toString()));
+		action->setText(i18n("Hide Column '%1'", d->mFilterModel.headerData(index, Qt::Horizontal, Qt::DisplayRole).toString()));
 		menu->addAction(action);
 		if(d->mUi->treeView->header()->sectionsHidden()) {
 			menu->addSeparator();
 		}
 	}
-	
+
 
 	if(d->mUi->treeView->header()->sectionsHidden()) {
 		int num_headings = d->mFilterModel.columnCount();
 		for(int i = 0; i < num_headings; ++i) {
 			if(d->mUi->treeView->header()->isSectionHidden(i)) {
 				action = new QAction(menu);
-				action->setText(i18n("Show column '%1'", d->mFilterModel.headerData(i, Qt::Horizontal, Qt::DisplayRole).toString()));
+				action->setText(i18n("Show Column '%1'", d->mFilterModel.headerData(i, Qt::Horizontal, Qt::DisplayRole).toString()));
 				action->setData(i); //We set data to be negative (and minus 1) to hide a column, and positive to show a column
 				menu->addAction(action);
 			}
@@ -487,6 +551,11 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
 	QAction *actionKB = NULL;
 	QAction *actionMB = NULL;
 	QAction *actionGB = NULL;
+	QAction *actionPercentage = NULL;
+	QAction *actionShowCmdlineOptions = NULL;
+	QAction *actionShowTooltips = NULL;
+	QAction *actionNormalizeCPUUsage = NULL;
+	
 
 	if( index == ProcessModel::HeadingVmSize || index == ProcessModel::HeadingMemory ||  index == ProcessModel::HeadingSharedMemory) {
 		//If the user right clicks on a column that contains a memory size, show a toggle option for displaying
@@ -494,20 +563,26 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
 		menu->addSeparator()->setText(i18n("Display Units"));
 		QActionGroup *unitsGroup = new QActionGroup(menu);
 		actionKB = new QAction(menu);
-		actionKB->setText(i18n("KiloBytes"));
+		actionKB->setText(i18n("Kilobytes"));
 		actionKB->setCheckable(true);
 		menu->addAction(actionKB);
 		unitsGroup->addAction(actionKB);
 		actionMB = new QAction(menu);
-		actionMB->setText(i18n("MegaBytes"));
+		actionMB->setText(i18n("Megabytes"));
 		actionMB->setCheckable(true);
 		menu->addAction(actionMB);
 		unitsGroup->addAction(actionMB);
 		actionGB = new QAction(menu);
-		actionGB->setText(i18n("GigaBytes"));
+		actionGB->setText(i18n("Gigabytes"));
 		actionGB->setCheckable(true);
 		menu->addAction(actionGB);
 		unitsGroup->addAction(actionGB);
+		actionPercentage = new QAction(menu);
+		actionPercentage->setText(i18n("Percentage"));
+		actionPercentage->setCheckable(true);
+		menu->addAction(actionPercentage);
+		unitsGroup->addAction(actionPercentage);
+
 		unitsGroup->setExclusive(true);
 		switch(d->mModel.units()) {
 		  case ProcessModel::UnitsKB:
@@ -519,8 +594,31 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
 		  case ProcessModel::UnitsGB:
 			actionGB->setChecked(true);
 			break;
+		  case ProcessModel::UnitsPercentage:
+			actionPercentage->setChecked(true);
+			break;
 		}
+	} else if(index == ProcessModel::HeadingName) {
+		menu->addSeparator();
+		actionShowCmdlineOptions = new QAction(menu);
+		actionShowCmdlineOptions->setText(i18n("Display command line options"));
+		actionShowCmdlineOptions->setCheckable(true);
+		actionShowCmdlineOptions->setChecked(d->mModel.isShowCommandLineOptions());
+		menu->addAction(actionShowCmdlineOptions);
+	} else if(index == ProcessModel::HeadingCPUUsage) {
+		menu->addSeparator();
+		actionNormalizeCPUUsage = new QAction(menu);
+		actionNormalizeCPUUsage->setText(i18n("Divide CPU usage by number of CPUs"));
+		actionNormalizeCPUUsage->setCheckable(true);
+		actionNormalizeCPUUsage->setChecked(d->mModel.isNormalizedCPUUsage());
+		menu->addAction(actionNormalizeCPUUsage);
 	}
+	menu->addSeparator();
+	actionShowTooltips = new QAction(menu);
+	actionShowTooltips->setCheckable(true);
+	actionShowTooltips->setChecked(d->mModel.isShowingTooltips());
+	actionShowTooltips->setText(i18n("Show Tooltips"));
+	menu->addAction(actionShowTooltips);
 
 
 	QAction *result = menu->exec(d->mUi->treeView->header()->mapToGlobal(point));
@@ -534,7 +632,20 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
 	} else if(result == actionGB) {
 		d->mModel.setUnits(ProcessModel::UnitsGB);
 		return;
+	} else if(result == actionPercentage) {
+		d->mModel.setUnits(ProcessModel::UnitsPercentage);
+		return;
+	} else if(result == actionShowCmdlineOptions) {
+		d->mModel.setShowCommandLineOptions(actionShowCmdlineOptions->isChecked());
+		return;
+	} else if(result == actionNormalizeCPUUsage) {
+		d->mModel.setNormalizedCPUUsage(actionNormalizeCPUUsage->isChecked());
+		return;
+	} else if(result == actionShowTooltips) {
+		d->mModel.setShowingTooltips(actionShowTooltips->isChecked());
+		return;
 	}
+
 	int i = result->data().toInt();
 	//We set data to be negative to hide a column, and positive to show a column
 	if(i < 0)
@@ -544,12 +655,12 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
 		d->mUi->treeView->resizeColumnToContents(i);
 		d->mUi->treeView->resizeColumnToContents(d->mFilterModel.columnCount());
 	}
-	menu->deleteLater();	
+	menu->deleteLater();
 }
 
-void KSysGuardProcessList::expandAllChildren(const QModelIndex &parent) 
+void KSysGuardProcessList::expandAllChildren(const QModelIndex &parent)
 {
-	//This is called when the user expands a node.  This then expands all of its 
+	//This is called when the user expands a node.  This then expands all of its
 	//children.  This will trigger this function again recursively.
 	QModelIndex sourceParent = d->mFilterModel.mapToSource(parent);
 	for(int i = 0; i < d->mModel.rowCount(sourceParent); i++) {
@@ -608,11 +719,11 @@ void KSysGuardProcessList::showEvent ( QShowEvent * event )  //virtual protected
 	if(!d->mUpdateTimer->isActive()) {
 		d->mUpdateTimer->start(d->mUpdateIntervalMSecs);
 	}
-	
+
 	QWidget::showEvent(event);
 }
 
-void KSysGuardProcessList::changeEvent( QEvent * event ) 
+void KSysGuardProcessList::changeEvent( QEvent * event )
 {
 	if (event->type() == QEvent::LanguageChange) {
 		d->mModel.retranslateUi();
@@ -637,15 +748,16 @@ void KSysGuardProcessList::updateList()
 	if(isVisible()) {
 		d->mModel.update(d->mUpdateIntervalMSecs);
 		d->mUpdateTimer->start(d->mUpdateIntervalMSecs);
+		emit updated();
 	}
 }
 
-int KSysGuardProcessList::updateIntervalMSecs() const 
+int KSysGuardProcessList::updateIntervalMSecs() const
 {
 	return d->mUpdateIntervalMSecs;
-}	
+}
 
-void KSysGuardProcessList::setUpdateIntervalMSecs(int intervalMSecs) 
+void KSysGuardProcessList::setUpdateIntervalMSecs(int intervalMSecs)
 {
 	d->mUpdateIntervalMSecs = intervalMSecs;
 	d->mUpdateTimer->setInterval(d->mUpdateIntervalMSecs);
@@ -662,15 +774,16 @@ bool KSysGuardProcessList::reniceProcesses(const QList<long long> &pids, int nic
 	}
 	if(unreniced_pids.isEmpty()) return true; //All processes were reniced successfully
 	if(!d->mModel.isLocalhost()) return false; //We can't use kdesu to renice non-localhost processes
-	
+
 	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
 	arguments << "--" << "renice" << QString::number(niceValue);
 
         for (int i = 0; i < unreniced_pids.size(); ++i) {
 		arguments << QString::number(unreniced_pids.at(i));
 	}
 
-	QString su = KStandardDirs::findExe("kdesu");
+	QString su = KStandardDirs::findExe("kdesu");  //kdesu is a libexec program, so it will not be in the path.  findExe will find it correctly anyway
 	if(su.isEmpty()) return false;  //Cannot find kdesu
 
 	QProcess *reniceProcess = new QProcess(NULL);
@@ -696,10 +809,10 @@ void KSysGuardProcessList::reniceSelectedProcesses()
 {
 	QList<KSysGuard::Process *> processes = selectedProcesses();
 	QStringList selectedAsStrings;
-	
+
 	if (processes.isEmpty())
 	{
-		KMessageBox::sorry(this, i18n("You need to select a process first."));
+		KMessageBox::sorry(this, i18n("You must select a process first."));
 		return;
 	}
 
@@ -782,9 +895,9 @@ void KSysGuardProcessList::reniceSelectedProcesses()
 
 	}
 	if(!renicePids.isEmpty()) {
-		Q_ASSERT(reniceDlg.newCPUPriority <= 20 && reniceDlg.newCPUPriority >= -20); 
+		Q_ASSERT(reniceDlg.newCPUPriority <= 20 && reniceDlg.newCPUPriority >= -20);
 		if(!reniceProcesses(renicePids, reniceDlg.newCPUPriority)) {
-			KMessageBox::sorry(this, i18n("You do not have sufficient privileges to change the CPU priority.  Aborting"));
+			KMessageBox::sorry(this, i18n("You do not have sufficient privileges to change the CPU priority. Aborting."));
 			return;
 		}
 	}
@@ -816,8 +929,9 @@ bool KSysGuardProcessList::changeIoScheduler(const QList< long long> &pids, KSys
 	if(su.isEmpty()) return false;  //Cannot find kdesu
 
 	//We must use kdesu to kill the process
-	
+
 	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
 	if(unchanged_pids.size() == 1) {
 		arguments << "--" << "ionice" << "-p" << QString::number(unchanged_pids.at(0)) << "-c";
 		switch(newIoSched) {
@@ -880,9 +994,47 @@ bool KSysGuardProcessList::changeCpuScheduler(const QList< long long> &pids, KSy
 		}
 	}
 	if(unchanged_pids.isEmpty()) return true;
-	if(!d->mModel.isLocalhost()) return false; //We can't use kdesu to kill non-localhost processes
+	if(!d->mModel.isLocalhost()) {
+		KMessageBox::sorry(this, i18n("No."));
+		return false; //We can't use kdesu to kill non-localhost processes
+	}
 
-	return false;  //TODO Must implement some form of kdesu or something
+	QString su = KStandardDirs::findExe("kdesu");
+	if(su.isEmpty()) {
+		KMessageBox::sorry(this, i18n("Could not find kdesu executable."));
+		return false;  //Cannot find kdesu
+	}
+	QString setscheduler = KStandardDirs::findExe("setscheduler");
+	if(setscheduler.isEmpty()) {
+		KMessageBox::sorry(this, i18n("Could not find setscheduler executable.  This should have been installed alongside system monitor."));
+		return false;
+	}
+
+	//We must use kdesu to kill the process
+
+	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
+	if(unchanged_pids.size() == 1) {
+		arguments << "--" << setscheduler << QString::number(unchanged_pids.at(0)) << QString::number((int)newCpuSched) << QString::number(newCpuSchedPriority);
+	} else {
+		//Cope with multiple pids by doing a for loop
+		arguments << "--" << "sh" << "-c";
+		QString sh("for f in ");
+
+	        for (int i = 0; i < unchanged_pids.size(); ++i) {
+			sh += QString::number(unchanged_pids.at(i)) + " ";
+		}
+		sh += "; do " + setscheduler + "\"$f\" " + newCpuSched + " " + newCpuSchedPriority;
+		sh += "; done";
+
+		arguments << sh;
+	}
+
+	QProcess *process = new QProcess(NULL);
+	connect(process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(ioniceFailed()));
+	connect(process, SIGNAL(finished( int, QProcess::ExitStatus) ), this, SLOT(updateList()));
+	process->start(su, arguments);
+	return true;  //assume it ran successfully :(  We cannot seem to actually check if it did.  There must be a better solution
 }
 
 bool KSysGuardProcessList::killProcesses(const QList< long long> &pids, int sig)
@@ -900,8 +1052,10 @@ bool KSysGuardProcessList::killProcesses(const QList< long long> &pids, int sig)
 	QString su = KStandardDirs::findExe("kdesu");
 	if(su.isEmpty()) return false;  //Cannot find kdesu
 
+	kDebug() << "running " << su;
 	//We must use kdesu to kill the process
 	QStringList arguments;
+	arguments << "--attach" << QString::number(window()->winId()) << "--noignorebutton";
 	arguments << "--" << "kill";
 	if(sig != SIGTERM) {
 		arguments << ('-' + QString::number(sig));
@@ -910,7 +1064,7 @@ bool KSysGuardProcessList::killProcesses(const QList< long long> &pids, int sig)
         for (int i = 0; i < unkilled_pids.size(); ++i) {
 		arguments << QString::number(unkilled_pids.at(i));
 	}
-	
+
 	QProcess *killProcess = new QProcess(NULL);
 	connect(killProcess, SIGNAL(error(QProcess::ProcessError)), this, SLOT(killFailed()));
 	connect(killProcess, SIGNAL(finished( int, QProcess::ExitStatus) ), this, SLOT(updateList()));
@@ -929,21 +1083,22 @@ void KSysGuardProcessList::killSelectedProcesses()
 		selectedPids << process->pid;
 		selectedAsStrings << d->mModel.getStringForProcess(process);
 	}
-	
+
 	if (selectedAsStrings.isEmpty())
 	{
-		KMessageBox::sorry(this, i18n("You need to select a process first."));
+		KMessageBox::sorry(this, i18n("You must select a process first."));
 		return;
 	}
 	else
 	{
-		QString  msg = i18np("Do you want to kill the selected process?",
-				"Do you want to kill the %1 selected processes?",
-				selectedAsStrings.count());
+		int count = selectedAsStrings.count();
+                QString  msg = i18np("Are you sure you want to kill this process?",
+				"Are you sure you want to kill these %1 processes?",
+				count);
 
 		int res = KMessageBox::warningContinueCancelList(this, msg, selectedAsStrings,
-								 i18n("Kill Process"),
-								 KGuiItem(i18n("Kill")),
+								 i18np("Kill process", "Kill %1 processes", count),
+								 KGuiItem(i18n("Kill"), "edit-bomb"),
 								 KStandardGuiItem::cancel(),
 								 "killconfirmation");
 		if (res != KMessageBox::Continue)
@@ -964,17 +1119,17 @@ void KSysGuardProcessList::killSelectedProcesses()
 void KSysGuardProcessList::reniceFailed()
 {
 	KMessageBox::sorry(this, i18n("You do not have the permission to renice the process and there "
-					"was a problem trying to run as root"));
+					"was a problem trying to run as root."));
 }
 void KSysGuardProcessList::killFailed()
 {
 	KMessageBox::sorry(this, i18n("You do not have the permission to kill the process and there "
-					"was a problem trying to run as root"));
+					"was a problem trying to run as root."));
 }
 void KSysGuardProcessList::ioniceFailed()
 {
 	KMessageBox::sorry(this, i18n("You do not have the permission to set the I/O priority and there "
-					"was a problem trying to run as root"));
+					"was a problem trying to run as root."));
 }
 bool KSysGuardProcessList::showTotals() const {
 	return d->mModel.showTotals();
@@ -996,6 +1151,9 @@ void KSysGuardProcessList::setUnits(ProcessModel::Units unit) {
 void KSysGuardProcessList::saveSettings(KConfigGroup &cg) {
 	/* Note that the ksysguard program does not use these functions.  It saves the settings itself to an xml file instead */
 	cg.writeEntry("units", (int)(units()));
+	cg.writeEntry("showCommandLineOptions", d->mModel.isShowCommandLineOptions());
+	cg.writeEntry("normalizeCPUUsage", d->mModel.isNormalizedCPUUsage());
+	cg.writeEntry("showTooltips", d->mModel.isShowingTooltips());
 	cg.writeEntry("showTotals", showTotals());
 	cg.writeEntry("filterState", (int)(state()));
 	cg.writeEntry("updateIntervalMSecs", updateIntervalMSecs());
@@ -1008,6 +1166,9 @@ void KSysGuardProcessList::saveSettings(KConfigGroup &cg) {
 void KSysGuardProcessList::loadSettings(const KConfigGroup &cg) {
 	/* Note that the ksysguard program does not use these functions.  It saves the settings itself to an xml file instead */
 	setUnits((ProcessModel::Units) cg.readEntry("units", (int)ProcessModel::UnitsKB));
+	d->mModel.setShowCommandLineOptions(cg.readEntry("showCommandLineOptions", false));
+	d->mModel.setNormalizedCPUUsage(cg.readEntry("normalizeCPUUsage", true));
+	d->mModel.setShowingTooltips(cg.readEntry("showTooltips", true));
 	setShowTotals(cg.readEntry("showTotals", true));
 	setStateInt(cg.readEntry("filterState", (int)ProcessFilter::AllProcesses));
 	setUpdateIntervalMSecs(cg.readEntry("updateIntervalMSecs", 2000));
@@ -1049,3 +1210,13 @@ bool KSysGuardProcessList::eventFilter(QObject *obj, QEvent *event) {
 ProcessModel *KSysGuardProcessList::processModel() {
 	return &d->mModel;
 }
+
+void KSysGuardProcessList::setKillButtonVisible(bool visible)
+{
+	d->mUi->btnKillProcess->setVisible(visible);
+}
+
+bool KSysGuardProcessList::isKillButtonVisible() const
+{
+	return d->mUi->btnKillProcess->isVisible();
+}
