@@ -28,6 +28,7 @@
 #include <QGraphicsSceneHoverEvent>
 #include <QTimer>
 #include <QX11Info>
+#include <QDBusInterface>
 
 #include <KDialog>
 #include <KColorScheme>
@@ -50,24 +51,28 @@
 
 #include <taskmanager/task.h>
 
-const int FAST_UPDATE_DELAY = 200;
+const int FAST_UPDATE_DELAY = 100;
 const int UPDATE_DELAY = 500;
 const int DRAG_SWITCH_DELAY = 1000;
 
 Pager::Pager(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args),
       m_displayedText(None),
+      m_currentDesktopSelected(DoNothing),
       m_showWindowIcons(false),
       m_showOwnBackground(false),
       m_rows(2),
       m_columns(0),
+      m_desktopDown(false),
       m_hoverIndex(-1),
       m_colorScheme(0),
+      m_verticalFormFactor(false),
       m_dragId(0),
       m_dirtyDesktop(-1),
       m_dragStartDesktop(-1),
       m_dragHighlightedDesktop(-1),
-      m_dragSwitchDesktop(-1)
+      m_dragSwitchDesktop(-1),
+      m_ignoreNextSizeConstraint(false)
 {
     setAcceptsHoverEvents(true);
     setAcceptDrops(true);
@@ -91,11 +96,11 @@ void Pager::init()
     m_displayedText = (DisplayedText)cg.readEntry("displayedText", (int)m_displayedText);
     m_showWindowIcons = cg.readEntry("showWindowIcons", m_showWindowIcons);
     m_rows = globalConfig().readEntry("rows", m_rows);
+    m_verticalFormFactor = (formFactor() == Plasma::Vertical);
+    m_currentDesktopSelected = (CurrentDesktopSelected)cg.readEntry("currentDesktopSelected", (int)m_currentDesktopSelected);
 
     if (m_rows < 1) {
         m_rows = 1;
-    } else if (m_rows > m_desktopCount) {
-        m_rows = m_desktopCount;
     }
 
     m_timer = new QTimer(this);
@@ -140,6 +145,30 @@ Pager::~Pager()
 
 void Pager::constraintsEvent(Plasma::Constraints constraints)
 {
+    // whenever we switch to/from vertical form factor, swap the rows and columns around
+    if (constraints & Plasma::FormFactorConstraint) {
+        if (m_verticalFormFactor != (formFactor() == Plasma::Vertical) && m_columns != m_rows) {
+            int temp = m_columns;
+            m_columns = m_rows;
+            m_rows = temp;
+
+            // save this new value in the global config
+            KConfigGroup globalcg = globalConfig();
+            if (m_rows > m_desktopCount) {
+              m_rows = m_desktopCount;
+            }
+            globalcg.writeEntry("rows", m_rows);
+            configNeedsSaving();
+            // no need to recalculate everything twice
+            if (!(constraints & Plasma::SizeConstraint)) {
+                recalculateGeometry();
+                recalculateWindowRects();
+                update();
+            }
+        }
+        m_verticalFormFactor = (formFactor() == Plasma::Vertical);
+    }
+
     if (constraints & Plasma::SizeConstraint) {
         recalculateGeometry();
         recalculateWindowRects();
@@ -148,13 +177,17 @@ void Pager::constraintsEvent(Plasma::Constraints constraints)
             m_background->resizeFrame(size());
         }
     }
+
     if (constraints & Plasma::FormFactorConstraint) {
         if (formFactor() == Plasma::Horizontal) {
             setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
+            setMinimumSize(preferredSize().width(), 0);
         } else if (formFactor() == Plasma::Vertical) {
             setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+            setMinimumSize(0, preferredSize().height());
         } else {
             setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+            setMinimumSize(preferredSize());
         }
     }
 }
@@ -210,26 +243,37 @@ void Pager::createConfigurationInterface(KConfigDialog *parent)
     }
 
     ui.showWindowIconsCheckBox->setChecked(m_showWindowIcons);
-    ui.spinRows->setValue(m_rows);
+    if (formFactor() == Plasma::Vertical) {
+        ui.labelRows->setText(i18n("Number of columns:"));
+        ui.spinRows->setValue(m_columns);
+    } else {
+        ui.spinRows->setValue(m_rows);
+    }
     ui.spinRows->setMaximum(m_desktopCount);
+
+    switch (m_currentDesktopSelected){
+        case DoNothing:
+            ui.doNothingRadioButton->setChecked(true);
+            break;
+
+        case ShowDesktop:
+            ui.showDesktopRadioButton->setChecked(true);
+            break;
+
+        case ShowDashboard:
+            ui.showDashboardRadioButton->setChecked(true);
+            break;
+    }
 }
 
 void Pager::recalculateGeometry()
 {
-    if (!m_rects.isEmpty() && geometry().size() == m_size) {
-        //kDebug() << "leaving because" << !m_rects.isEmpty() << " and " << contentSize() << "==" << m_size;
-        return;
-    }
-
     int padding = 2; // Space between miniatures of desktops
     int textMargin = 3; // Space between name of desktop and border
-    int columns = m_desktopCount / m_rows + m_desktopCount % m_rows;
-    int rows = m_rows;
-
-    //inverse rows and columns in vertical panel
-    if (formFactor() == Plasma::Vertical) {
-        rows = columns;
-        columns = m_rows;
+    int rows = qMax(qMin(m_rows, m_desktopCount), 1);
+    int columns = m_desktopCount / rows;
+    if (m_desktopCount % rows > 0) {
+        columns++;
     }
 
     qreal leftMargin = 0;
@@ -237,63 +281,87 @@ void Pager::recalculateGeometry()
     qreal rightMargin = 0;
     qreal bottomMargin = 0;
 
+    qreal ratio = (qreal)Kephal::ScreenUtils::desktopGeometry().width() / (qreal)Kephal::ScreenUtils::desktopGeometry().height();
+
     if (formFactor() == Plasma::Vertical || formFactor() == Plasma::Horizontal) {
         m_background->setElementPrefix(QString());
         m_background->getMargins(leftMargin, topMargin, rightMargin, bottomMargin);
 
-        qreal ratio = (qreal)Kephal::ScreenUtils::desktopGeometry().width() / (qreal)Kephal::ScreenUtils::desktopGeometry().height();
-
         if (formFactor() == Plasma::Vertical) {
-            qreal optimalSize = (geometry().width() - KIconLoader::SizeSmall*ratio * columns + padding*(columns-1)) / 2;
+            qreal optimalSize = (geometry().width() - KIconLoader::SizeSmall*ratio * columns - padding*(columns-1)) / 2;
 
             if (optimalSize < leftMargin || optimalSize < rightMargin) {
                 leftMargin = rightMargin = qMax(qreal(0), optimalSize);
                 m_showOwnBackground = false;
             }
         } else if (formFactor() == Plasma::Horizontal) {
-            qreal optimalSize = (geometry().height() - KIconLoader::SizeSmall*rows + padding*(rows-1)) / 2;
+            qreal optimalSize = (geometry().height() - KIconLoader::SizeSmall*rows - padding*(rows-1)) / 2;
 
             if (optimalSize < topMargin || optimalSize < bottomMargin) {
-                topMargin = bottomMargin =  qMax(qreal(0), optimalSize);
+                topMargin = bottomMargin = qMax(qreal(0), optimalSize);
                 m_showOwnBackground = false;
             }
-        } else {
-            m_showOwnBackground = true;
         }
     } else {
         getContentsMargins(&leftMargin, &topMargin, &rightMargin, &bottomMargin);
     }
 
+
     qreal itemHeight;
     qreal itemWidth;
+    qreal preferredItemHeight;
+    qreal preferredItemWidth;
 
     if (formFactor() == Plasma::Vertical) {
-        if (location() == Plasma::Floating)
-            itemWidth = (contentsRect().width() - padding * (columns - 1)) / columns;
-        else
-            itemWidth = (contentsRect().width() - leftMargin - rightMargin - padding * (columns - 1)) / columns;
+        // work out the preferred size based on the width of the contentsRect
+        preferredItemWidth = (contentsRect().width() - leftMargin - rightMargin - padding * (columns - 1)) / columns;
+        preferredItemHeight = preferredItemWidth / ratio;
+        // make sure items of the new size actually fit in the current contentsRect
+        itemHeight = (contentsRect().height() - topMargin - bottomMargin - padding * (rows - 1)) / rows;
+        if (itemHeight > preferredItemHeight) {
+            itemHeight = preferredItemHeight;
+        }
+        itemWidth = itemHeight * ratio;
+
         m_widthScaleFactor = itemWidth / Kephal::ScreenUtils::desktopGeometry().width();
-        itemHeight = Kephal::ScreenUtils::desktopGeometry().height() * m_widthScaleFactor;
-        m_heightScaleFactor = m_widthScaleFactor;
-    } else {
-        if (location() == Plasma::Floating)
-            itemHeight = (contentsRect().height() - padding * (rows - 1)) / rows;
-        else
-            itemHeight = (contentsRect().height() - topMargin - bottomMargin - padding * (rows - 1)) / rows;
         m_heightScaleFactor = itemHeight / Kephal::ScreenUtils::desktopGeometry().height();
-        itemWidth = Kephal::ScreenUtils::desktopGeometry().width() * m_heightScaleFactor;
+    } else {
+        // work out the preferred size based on the height of the contentsRect
+        if (formFactor() == Plasma::Horizontal) {
+            preferredItemHeight = (contentsRect().height() - topMargin - bottomMargin - padding * (rows - 1)) / rows;
+        } else {
+            preferredItemHeight = (contentsRect().height() - padding * (rows - 1)) / rows;
+        }
+        preferredItemWidth = preferredItemHeight * ratio;
+
         if (m_displayedText == Name) {
             // When containment is in this position we are not limited by low width and we can
             // afford increasing width of applet to be able to display every name of desktops
             for (int i = 0; i < m_desktopCount; i++) {
                 QFontMetricsF metrics(KGlobalSettings::taskbarFont());
                 QSizeF textSize = metrics.size(Qt::TextSingleLine, KWindowSystem::desktopName(i+1));
-                if (textSize.width() + textMargin * 2 > itemWidth) {
-                     itemWidth = textSize.width() + textMargin * 2;
+                if (textSize.width() + textMargin * 2 > preferredItemWidth) {
+                     preferredItemWidth = textSize.width() + textMargin * 2;
                 }
             }
         }
+
+        // make sure items of the new size actually fit in the current contentsRect
+        if (formFactor() == Plasma::Horizontal) {
+            itemWidth = (contentsRect().width() - leftMargin - rightMargin - padding * (columns - 1)) / columns;
+        } else {
+            itemWidth = (contentsRect().width() - padding * (columns - 1)) / columns;
+        }
+        if (itemWidth > preferredItemWidth) {
+            itemWidth = preferredItemWidth;
+        }
+        itemHeight = preferredItemHeight;
+        if (itemWidth < itemHeight * ratio) {
+            itemHeight = itemWidth / ratio;
+        }
+
         m_widthScaleFactor = itemWidth / Kephal::ScreenUtils::desktopGeometry().width();
+        m_heightScaleFactor = itemHeight / Kephal::ScreenUtils::desktopGeometry().height();
     }
 
     m_rects.clear();
@@ -326,19 +394,40 @@ void Pager::recalculateGeometry()
         m_background->resizeFrame(itemRect.size());
     }
 
-    m_size = QSizeF(ceil(columns * itemWidth + padding * (columns - 1) + leftMargin + rightMargin),
-                    ceil(rows * itemHeight + padding * (rows - 1) + topMargin + bottomMargin));
+    // allow ignored resizes only if the height has changed (or if the width has changed if we are in a vertical panel)
+    if (!m_ignoreNextSizeConstraint || (formFactor() != Plasma::Vertical && contentsRect().height() != m_size.height())
+                                    || (formFactor() == Plasma::Vertical && contentsRect().width()  != m_size.width())) {
 
-    //kDebug() << "new size set" << m_size << m_rows << m_columns << columns << itemWidth;
+        // this new size will have the same height/width as the horizontal/vertical panel has given it
+        QSizeF preferred = QSizeF(ceil(columns * preferredItemWidth + padding * (columns - 1) + leftMargin + rightMargin),
+                                  ceil(rows * preferredItemHeight + padding * (rows - 1) + topMargin + bottomMargin));
 
-    resize(m_size);
-    setPreferredSize(m_size);
-    if (m_desktopLayoutOwner && columns != m_columns) {
-        // must own manager selection before setting global desktop layout
+        //kDebug() << "current size:" << contentsRect() << " new preferred size: " << preferred << " form factor:" << formFactor() << " grid:" << m_rows << "x" << m_columns <<
+        //            " actual grid:" << rows << "x" << columns << " item size:" << itemWidth << "x" << itemHeight << " preferred item size:" << preferredItemWidth << "x" << preferredItemHeight;
+
+        // make sure the minimum size is smaller than preferred
+        setMinimumSize(qMin(preferred.width(),  minimumSize().width()),
+                       qMin(preferred.height(), minimumSize().height()));
+        //resize(m_size);
+        setPreferredSize(preferred);
+
+        m_ignoreNextSizeConstraint = true;
+    } else {
+        m_ignoreNextSizeConstraint = false;
+        //kDebug() << "ignoring event - current size:" << contentsRect() << " form factor:" << formFactor() << " grid:" << m_rows << "x" << m_columns <<
+        //            " actual grid:" << rows << "x" << columns << " item size:" << itemWidth << "x" << itemHeight << " preferred item size:" << preferredItemWidth << "x" << preferredItemHeight;
+    }
+
+    m_size = contentsRect().size();
+
+    if (columns != m_columns) {
         m_columns = columns;
-        NET::Orientation orient = NET::OrientationHorizontal;
-        NETRootInfo i( QX11Info::display(), 0 );
-        i.setDesktopLayout( orient, columns, rows, NET::DesktopLayoutCornerTopLeft );
+        if (m_desktopLayoutOwner) {
+            // must own manager selection before setting global desktop layout
+            NET::Orientation orient = NET::OrientationHorizontal;
+            NETRootInfo i( QX11Info::display(), 0 );
+            i.setDesktopLayout( orient, columns, rows, NET::DesktopLayoutCornerTopLeft );
+        }
     }
 }
 
@@ -349,10 +438,14 @@ void Pager::recalculateWindowRects()
     for (int i = 0; i < m_desktopCount; i++) {
         m_windowRects.append(QList<QPair<WId, QRect> >());
     }
+
     m_activeWindows.clear();
     m_windowInfo.clear();
-    foreach(WId window, windows) {
-        KWindowInfo info = KWindowSystem::windowInfo(window, NET::WMGeometry | NET::WMFrameExtents | NET::WMWindowType | NET::WMDesktop | NET::WMState | NET::XAWMState | NET::WMVisibleName);
+
+    foreach (WId window, windows) {
+        KWindowInfo info = KWindowSystem::windowInfo(window, NET::WMGeometry | NET::WMFrameExtents |
+                                                             NET::WMWindowType | NET::WMDesktop |
+                                                             NET::WMState | NET::XAWMState | NET::WMVisibleName);
         NET::WindowType type = info.windowType(NET::NormalMask | NET::DialogMask | NET::OverrideMask |
                                                NET::UtilityMask | NET::DesktopMask | NET::DockMask |
                                                NET::TopMenuMask | NET::SplashMask | NET::ToolbarMask |
@@ -401,13 +494,12 @@ void Pager::configAccepted()
 
     DisplayedText displayedText;
 
-    if (ui.desktopNumberRadioButton->isChecked()){
+    if (ui.desktopNumberRadioButton->isChecked()) {
         displayedText = Number;
 
-    }else if (ui.desktopNameRadioButton->isChecked()){
+    } else if (ui.desktopNameRadioButton->isChecked()) {
         displayedText = Name;
-
-    }else{
+    } else {
         displayedText = None;
     }
 
@@ -423,12 +515,38 @@ void Pager::configAccepted()
         changed = true;
     }
 
+    CurrentDesktopSelected currentDesktopSelected;
+
+    if (ui.doNothingRadioButton->isChecked()) {
+        currentDesktopSelected = DoNothing;
+    } else if (ui.showDesktopRadioButton->isChecked()) {
+        currentDesktopSelected = ShowDesktop;
+    } else {
+        currentDesktopSelected = ShowDashboard;
+    }
+
+    if ((int)m_currentDesktopSelected != (int)currentDesktopSelected) {
+        m_currentDesktopSelected = currentDesktopSelected;
+        cg.writeEntry("currentDesktopSelected", (int)m_currentDesktopSelected);
+        changed = true;
+    }
+
     // we need to keep all pager applets consistent since this affects
     // the layout of the desktops as used by the window manager,
     // so we store the row count in the applet global configuration
-    if (m_rows != ui.spinRows->value()) {
+    int rows = 0;
+    if (formFactor() == Plasma::Vertical) {
+        rows = m_desktopCount / ui.spinRows->value();
+        if (m_desktopCount % ui.spinRows->value() > 0) {
+            rows++;
+        }
+    } else {
+        rows = ui.spinRows->value();
+    }
+
+    if (m_rows != rows) {
         KConfigGroup globalcg = globalConfig();
-        m_rows = ui.spinRows->value();
+        m_rows = rows;
         if (m_rows > m_desktopCount) {
             m_rows = m_desktopCount;
         }
@@ -455,6 +573,7 @@ void Pager::currentDesktopChanged(int desktop)
 
     m_currentDesktop = desktop;
     m_dirtyDesktop = -1;
+    m_desktopDown = false;
 
     if (!m_timer->isActive()) {
         m_timer->start(FAST_UPDATE_DELAY);
@@ -506,10 +625,6 @@ void Pager::numberOfDesktopsChanged(int num)
     m_dirtyDesktop = -1;
     m_desktopCount = num;
 
-    if (m_rows > m_desktopCount) {
-        m_rows = m_desktopCount;
-    }
-
     m_rects.clear();
     recalculateGeometry();
     recalculateWindowRects();
@@ -698,6 +813,20 @@ void Pager::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
         // only change the desktop if the user presses and releases the mouse on the same desktop
         KWindowSystem::setCurrentDesktop(m_dragStartDesktop + 1);
         m_currentDesktop = m_dragStartDesktop + 1;
+    } else if (m_dragStartDesktop != -1 && m_dragStartDesktop < m_rects.size() &&
+               m_rects[m_dragStartDesktop].contains(event->pos()) &&
+               m_currentDesktop == m_dragStartDesktop + 1) {
+        // toogle the desktop or the dashboard
+        // if the user presses and releases the mouse on the current desktop, default option is do nothing
+        if (m_currentDesktopSelected == ShowDesktop) {
+
+            NETRootInfo info(QX11Info::display(), 0);
+            m_desktopDown = !m_desktopDown;
+            info.setShowingDesktop(m_desktopDown);
+        } else if (m_currentDesktopSelected == ShowDashboard) {
+            QDBusInterface plasmaApp("org.kde.plasma-desktop", "/App");
+            plasmaApp.call("toggleDashboard");
+        }
     }
 
     m_dragId = 0;
@@ -862,7 +991,7 @@ void Pager::dropEvent(QGraphicsSceneDragDropEvent *event)
 void Pager::animationUpdate(qreal progress, int animId)
 {
     int i = 0;
-    foreach (AnimInfo anim, m_animations) {
+    foreach (const AnimInfo &anim, m_animations) {
         if (anim.animId == animId) {
             break;
         }
@@ -950,40 +1079,44 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
     }
 
     // Draw miniatures of windows from each desktop
-    painter->setPen(windowPen);
-    for (int i = 0; i < m_windowRects.count(); i++) {
-        for (int j = 0; j < m_windowRects[i].count(); j++) {
-            QRect rect = m_windowRects[i][j].second;
+    if (!m_rects.isEmpty() && m_rects[0].width() > 12 && m_rects[0].height() > 12) {
+        painter->setPen(windowPen);
+        for (int i = 0; i < m_windowRects.count(); i++) {
+            for (int j = 0; j < m_windowRects[i].count(); j++) {
+                QRect rect = m_windowRects[i][j].second;
 
-            if (m_currentDesktop > 0 &&
-                m_currentDesktop <= m_rects.count() &&
-                m_rects[m_currentDesktop-1].contains(rect)) {
-                if (m_activeWindows.contains(rect)) {
-                    painter->setBrush(activeWindowBrushActiveDesk);
-                    painter->setPen(activeWindowPen);
+                if (m_currentDesktop > 0 &&
+                        m_currentDesktop <= m_rects.count() &&
+                        m_rects[m_currentDesktop-1].contains(rect)) {
+                    if (m_activeWindows.contains(rect)) {
+                        painter->setBrush(activeWindowBrushActiveDesk);
+                        painter->setPen(activeWindowPen);
+                    } else {
+                        painter->setBrush(windowBrushActiveDesk);
+                        painter->setPen(windowPen);
+                    }
                 } else {
-                    painter->setBrush(windowBrushActiveDesk);
-                    painter->setPen(windowPen);
+                    if (m_activeWindows.contains(rect)) {
+                        painter->setBrush(activeWindowBrush);
+                        painter->setPen(activeWindowPen);
+                    } else {
+                        painter->setBrush(windowBrush);
+                        painter->setPen(windowPen);
+                    }
                 }
-            } else {
-                if (m_activeWindows.contains(rect)) {
-                    painter->setBrush(activeWindowBrush);
-                    painter->setPen(activeWindowPen);
-                } else {
-                    painter->setBrush(windowBrush);
-                    painter->setPen(windowPen);
+                if (m_dragId == m_windowRects[i][j].first) {
+                    rect.translate((m_dragCurrentPos - m_dragOriginalPos).toPoint());
+                    painter->setClipRect(option->exposedRect);
+                } else if (i < m_rects.count()) {
+                    painter->setClipRect(m_rects[i].adjusted(1, 1, -1, -1));
+                }
+
+                painter->drawRect(rect);
+
+                if ((rect.width() > 16) && (rect.height() > 16) && m_showWindowIcons) {
+                    painter->drawPixmap(rect.x() + (rect.width() - 16) / 2, rect.y() + (rect.height() - 16) / 2, 16, 16,
+                            KWindowSystem::icon(m_windowRects[i][j].first, 16, 16, true));
                 }
-            }
-            if (m_dragId == m_windowRects[i][j].first) {
-                rect.translate((m_dragCurrentPos - m_dragOriginalPos).toPoint());
-                painter->setClipRect(option->exposedRect);
-            } else if (i < m_rects.count()) {
-                painter->setClipRect(m_rects[i].adjusted(1, 1, -1, -1));
-            }
-            painter->drawRect(rect);
-            if ((rect.width() > 16) && (rect.height() > 16) && m_showWindowIcons){
-                painter->drawPixmap(rect.x() + (rect.width() - 16) / 2, rect.y() + (rect.height() - 16) / 2, 16, 16,
-                                    KWindowSystem::icon(m_windowRects[i][j].first, 16, 16, true));
             }
         }
     }
@@ -1057,8 +1190,13 @@ void Pager::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *op
             QRectF target = m_rects[i];
             //take also shadow position and radius into account
             //kDebug() << target << result.height();
-            if (target.height() >= result.height() - radius * 2) {
-                QPointF paintPoint = target.center() - (result.rect().center() + QPoint(radius, radius)); 
+
+            // for the default size of the panel we can allow this "one pixel"
+            // offset on the bottom. the applet is so small that you almost
+            // can't see the offset and this brings back the labels for the
+            // panel's default size.
+            if (target.height() + 1 >= result.height() - radius * 2) {
+                QPointF paintPoint = target.center() - (result.rect().center() + QPoint(radius, radius));
 
                 if (paintPoint.x() + radius < target.x() + 1) {
                     paintPoint.setX(target.x() + 1 - radius);
@@ -1121,7 +1259,7 @@ void Pager::updateToolTip()
 
     QList<WId> windows;
 
-    foreach(KWindowInfo winInfo, m_windowInfo){
+    foreach(const KWindowInfo &winInfo, m_windowInfo){
         if (winInfo.isOnDesktop(hoverDesktopNumber) && !windows.contains(winInfo.win())) {
             bool active = (winInfo.win() == KWindowSystem::activeWindow());
             if ((taskCounter < 4) || active){    
