@@ -1,5 +1,7 @@
 /*
 *   Copyright 2007 by Aaron Seigo <aseigo@kde.org>
+*   Copyright 2008 by Alexis Ménard <darktears31@gmail.com>
+*
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2,
@@ -28,15 +30,11 @@
 #include <KDialog>
 #include <KIcon>
 
-#include <plasma/containment.h>
-#include <plasma/phase.h>
-#include <plasma/widgets/widget.h>
+#include <Plasma/Containment>
+#include <Plasma/Animator>
 
-#include "iconloader.h"
+#include "desktoplayout.h"
 
-#include "renderthread.h"
-
-class BackgroundDialog;
 class QAction;
 class QTimeLine;
 
@@ -46,11 +44,6 @@ namespace Plasma
     class Svg;
 }
 
-namespace Ui
-{
-    class config;
-}
-
 /*class Tool : public QObject, public QGraphicsItem
 {
     Q_OBJECT
@@ -69,66 +62,41 @@ class DefaultDesktop : public Plasma::Containment
 public:
     DefaultDesktop(QObject *parent, const QVariantList &args);
     ~DefaultDesktop();
-    void init();
-    void constraintsUpdated(Plasma::Constraints constraints);
+    void constraintsEvent(Plasma::Constraints constraints);
 
-    QList<QAction*> contextActions();
+    QList<QAction*> contextualActions();
 
-    /**
-     * Paints a default background image. Nothing fancy, but that's what plugins
-     * are for. Reimplemented from Plasma::Containment;
-     */
-    void paintInterface(QPainter *painter,
-                        const QStyleOptionGraphicsItem *option,
-                        const QRect& contentsRect);
+protected:
+    void dropEvent(QGraphicsSceneDragDropEvent *event);
 
 protected Q_SLOTS:
     void runCommand();
-    void configure();
-    void applyConfig();
-    void nextSlide(bool skipUpdates=false);
-    
-    void toggleDesktopImmutability();
+
     void lockScreen();
     void logout();
 
-    void updateBackground();
-    void updateBackground(int, const QImage &img);
-private:
-    void reloadConfig(bool skipUpdates=false);
-    QSize resolution() const;
-    void suspendStartup(bool suspend); // for ksmserver
+    void addPanel();
+    void addPanel(const QString &plugin);
 
+    void onAppletAdded(Plasma::Applet *, const QPointF &);
+    void onAppletRemoved(Plasma::Applet *);
+    void onAppletTransformedByUser();
+    void onAppletTransformedItself();
+    void refreshWorkingArea();
+
+private:
+    QMenu *m_addPanelsMenu;
     QAction *m_lockDesktopAction;
     QAction *m_appletBrowserAction;
+    QAction *m_addPanelAction;
     QAction *m_runCommandAction;
     QAction *m_setupDesktopAction;
     QAction *m_lockScreenAction;
     QAction *m_logoutAction;
     QAction *m_separator;
-    QAction *m_separator2;
 
-    BackgroundDialog *m_configDialog;
-
-    int m_backgroundMode;
-
-    // slideshow settings
-    // the index of which m_slidePath is currently visible
-    int m_currentSlide;
-    QTimer m_slideshowTimer;
-    QStringList m_slideFiles;
-
-    QPixmap m_bitmapBackground;
-    QString m_wallpaperPath;
-
-    int m_wallpaperPosition;
-    QColor m_wallpaperColor;
-
-    RenderThread m_renderer;
-    int m_current_renderer_token;
-
-    //Desktop icons
-    static IconLoader *s_icons;
+    DesktopLayout *m_layout;
+    bool dropping;
 };
 
 #endif // PLASMA_PANEL_H
